/***************************************************************************
                          kap_backendselector.cpp  -  description
                             -------------------
    begin                : Mon Jan 31 2000
    copyright            : (C) 2000 by jec
    email                : cuendet@linkvest.ch
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include <qbuttongroup.h>
#include <qradiobutton.h>
#include <qlayout.h>
#include <qframe.h>

#include <qmessagebox.h>
#include <klocale.h>

//#include "kups.h"
#include "kbuffer.h"
//#include "kap_parallelutil.h"
//#include "kap_manumodeldico.h"
#include "kap_backendselector.h"
#include "kprinterdescription.h"
#include "kap_ipputil.h"


Kap_BackendSelector::Kap_BackendSelector(QWidget *parent, const char *name )
:Kap_WizardPage(parent, name)
{
	setTitle(i18n("Backend selection"));

	group_ = new QButtonGroup(this);
	group_->hide();
//	QVBoxLayout* groupLayout = new QVBoxLayout(this);
//	groupLayout->addWidget(group_);

	QVBoxLayout* buttonLayout = new QVBoxLayout(this,5);

/*	parallel_ = new QRadioButton(this);
	parallel_->setChecked(true);
	parallel_->setText("Parallel port");
	group_->insert(parallel_);
	buttonLayout->addWidget(parallel_);

	serial_ = new QRadioButton(this);
	serial_->setText("Serial port");
	group_->insert(serial_);
	buttonLayout->addWidget(serial_);
*/
	local_ = new QRadioButton(this);
	local_->setText(i18n("Local printer (Parallel, serial, USB)"));
	local_->setChecked(true);
	group_->insert(local_);
	buttonLayout->addWidget(local_);

	lpd_ = new QRadioButton(this);
	lpd_->setText(i18n("LPD (Line Printer Deamon)"));
	group_->insert(lpd_);
	buttonLayout->addWidget(lpd_);

	ipp_ = new QRadioButton(this);
	ipp_->setText(i18n("IPP (Internet Printing Protocol)"));
	group_->insert(ipp_);
	buttonLayout->addWidget(ipp_);

	smb_ = new QRadioButton(this);
	smb_->setText(i18n("SMB (Windows shared printer)"));
	group_->insert(smb_);
	buttonLayout->addWidget(smb_);

	socket_ = new QRadioButton(this);
	socket_->setText(i18n("Socket (AppSocket network printer)"));
	group_->insert(socket_);
	buttonLayout->addWidget(socket_);

	file_ = new QRadioButton(this);
	file_->setText(i18n("File (Print to file)"));
	group_->insert(file_);
	buttonLayout->addWidget(file_);

	QFrame	*frame_ = new QFrame(this);
	frame_->setFrameStyle(QFrame::HLine|QFrame::Sunken);
	frame_->setLineWidth(1);
	frame_->setFixedHeight(10);
	buttonLayout->addWidget(frame_);

	class_ = new QRadioButton(this);
	class_->setText(i18n("Class of printers"));
	group_->insert(class_);
	buttonLayout->addWidget(class_);

	resize(200, 250);
	
	buttonLayout->activate();

	initialized_ = false;
}
Kap_BackendSelector::~Kap_BackendSelector()
{
}

bool Kap_BackendSelector::initialize()
{
	if (!initialized_)
	{
		int	flag = Kap_IppUtil::getAvailableBackends();
		if (flag)
		{
			if (!HAS_IPP(flag)) ipp_->setEnabled(false);
			if (!HAS_SOCKET(flag)) socket_->setEnabled(false);
			if (!HAS_LPD(flag)) lpd_->setEnabled(false);
			if (!HAS_SMB(flag)) smb_->setEnabled(false);
			initialized_ = true;
		}
	}
	return true;
}

void Kap_BackendSelector::enableClass(bool on)
{
	class_->setEnabled(on);
}

Kap_Main::e_PageType Kap_BackendSelector::getNextPageAfterDetection()
{
	return Kap_Main::ParallelSelector;
}

Kap_Main::e_PageType Kap_BackendSelector::getNextPage(QString&, bool fake)
{
	if (fake) return Kap_Main::LocalSelector;

	Kap_Main::e_PageType type = Kap_Main::Invalid;

	KPrinterDescription* descr = KPrinterDescription::get();
	
	descr->setURI(QString(""));
	if (class_->isChecked())
	{
		descr->setType(KPrinterManager::e_Class);
		type = Kap_Main::ClassSelector;
	}
	if (local_->isChecked())
	{
		type = Kap_Main::LocalSelector;
	}
	else if (lpd_->isChecked())
	{
		descr->setType(KPrinterManager::e_LPD);
		type = Kap_Main::LpdSelector;
	}
	else if (smb_->isChecked())
	{
		descr->setType(KPrinterManager::e_SMB);
		type = Kap_Main::LoginSelector;
	}
	else if (ipp_->isChecked())
	{
		descr->setType(KPrinterManager::e_IPP);
		type = Kap_Main::LoginSelector;
	}
	else if (socket_->isChecked())
	{
		descr->setType(KPrinterManager::e_Socket);
		type = Kap_Main::SocketSelector;
	}
	else if (file_->isChecked())
	{
		descr->setType(KPrinterManager::e_File);
		type = Kap_Main::FileSelector;
	}
	
	if (type == Kap_Main::Invalid)
	{
		QMessageBox::information(this,i18n("Add printer wizard"),i18n("This backend is not supported yet\nDrop me a line if you are interested \nto help in developing or testing it"),QMessageBox::Ok|QMessageBox::Default,0);
		type = Kap_Main::BackendSelector;
	}
	
	return type;
}

Kap_Main::e_PageType Kap_BackendSelector::getType() const
{
	return Kap_Main::BackendSelector;
}
