/***************************************************************************
                          kap_driverconfig.cpp  -  description
                             -------------------
    begin                : Tue Feb 8 2000
    copyright            : (C) 2000 by Michael Goffioul
    email                : goffioul@imec.be
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include <qlabel.h>
#include <qlayout.h>
#include <qpushbutton.h>
#include <qmessagebox.h>
#include <klocale.h>

#include "kap_driverconfig.h"
#include "kap_printerconfig.h"
#include "kprinterdescription.h"
#include "kap_manumodeldico.h"
#include "kprintermanager.h"
#include "kbuffer.h"
#include "kups_testpagegenerator.h"
#include "kprinter.h"


//void dump(global_ppd_file_t *ppd);

Kap_DriverConfigPage::Kap_DriverConfigPage(QWidget *parent, const char *name )
:Kap_WizardPage(parent,name)
{
	setTitle(i18n("Driver settings"));
        ppd_ = 0;

	QLabel	*l1 = new QLabel(this);
        l1->setText(i18n("<p>The selected driver will use a set of default values. If you want to change the printer settings, use the <b>Settings</b> button. To test your configuration, use the <b>Test</b> button.If you want to change the driver, use the <b>Prev</b> button.</p>"));

	QPushButton	*test_ = new QPushButton(i18n("Test..."), this);
        test_->setFixedSize(test_->sizeHint());
	QPushButton	*settings_ = new QPushButton(i18n("Settings..."), this);
        settings_->setFixedSize(settings_->sizeHint());

	connect(test_, SIGNAL(clicked()), SLOT(slotTest()));
        connect(settings_, SIGNAL(clicked()), SLOT(slotSettings()));

        manuLabel_ = new QLabel(this);
        modelLabel_ = new QLabel(this);
        driverLabel_ = new QLabel(this);
        QLabel	*l2 = new QLabel(i18n("<b>Manufacturer</b>:"), this);
        QLabel	*l3 = new QLabel(i18n("<b>Model name</b>:"), this);
        QLabel	*l4 = new QLabel(i18n("<b>Driver infos</b>:"), this);

        QVBoxLayout	*mainLayout_ = new QVBoxLayout(this, 10, 10), *labLayout_ = new QVBoxLayout(0, 0, 0);
        QHBoxLayout	*sub_ = new QHBoxLayout(0, 0, 5);
        QVBoxLayout	*subsub_ = new QVBoxLayout(0, 0, 0);
        QHBoxLayout	*btnLayout_ = new QHBoxLayout(0, 0, 10);
        mainLayout_->addWidget(l1,0);
        mainLayout_->addStretch(1);
        mainLayout_->addLayout(sub_, 0);
        sub_->addLayout(subsub_, 0);
        sub_->addLayout(labLayout_, 1);
        labLayout_->addWidget(manuLabel_, 0);
        labLayout_->addWidget(modelLabel_, 0);
        labLayout_->addWidget(driverLabel_, 0);
        subsub_->addWidget(l2, 0);
        subsub_->addWidget(l3, 0);
        subsub_->addWidget(l4, 0);
        mainLayout_->addLayout(btnLayout_, 0);
        mainLayout_->addStretch(1);
        btnLayout_->addWidget(test_, 0);
        btnLayout_->addWidget(settings_, 0);
        btnLayout_->addStretch(2);
}

Kap_DriverConfigPage::~Kap_DriverConfigPage()
{
}

bool Kap_DriverConfigPage::initialize()
{
	KPrinterDescription	*descr = KPrinterDescription::get();
        manuLabel_->setText(descr->getManufacturer());
        modelLabel_->setText(descr->getModel());
        driverLabel_->setText(descr->getNickName());
        if (ppd_)
        {
        }
	return true;
}

void Kap_DriverConfigPage::slotTest()
{
        // to test the printer, first create a temporary printer, then print a
        // test page, and finally remove the temporary printer.

	QString	errormsg;
        KPrinterManager	*mgr = KPrinterManager::get();

        // 1) load PPD file into a buffer
        KBuffer	*buffer = loadPPDAsBuffer();
        if (!buffer)
        {
	       	QMessageBox::critical(this, i18n("Error"), i18n("Unable to load driver %s !").arg(ppdFileName_), QMessageBox::Ok|QMessageBox::Default, 0);
                return;
        }

	// 2) update description structure with buffer and temp name
        QString	printername = QString("tempprinter_%1").arg(getuid());
        KPrinterDescription	*d = KPrinterDescription::get();
        d->setName(printername);
        d->setBuffer(buffer);

	// 3) create temporary printer
        if (!mgr->addPrinter(*d, false))
        {
	       	QMessageBox::critical(this, i18n("Error"), i18n("Creation of temporary printer failed.\n%1").arg(mgr->getLastError()), QMessageBox::Ok|QMessageBox::Default, 0);
                return;
        }
        KPrinter	*printer = mgr->getPrinterByName(printername);
        if (!printer)
        {
	       	QMessageBox::critical(this, i18n("Error"), i18n("Unable to find temporary printer !"), QMessageBox::Ok|QMessageBox::Default, 0);
                return;
        }
	// 4) send test page
        if (!Kups_TestPageGenerator::printPage(0, printer))
        	errormsg = i18n("Unable to send test page !");
	else
		QMessageBox::information(this, i18n("Printer test"), i18n("<p>The test page has been sent to the printer. Click the <b>OK</b> button when printing is finished.</p>"), QMessageBox::Ok|QMessageBox::Default, 0);

	// 5) remove temporary printer
	if (!printer->sendCommand(KPrinter::Remove))
		errormsg = i18n("Unable to remove temporary printer %1").arg(printer->getName());
	d->setBuffer(0);
        d->setName("");

	if (!errormsg.isEmpty())
	       	QMessageBox::critical(this, i18n("Error"), errormsg, QMessageBox::Ok|QMessageBox::Default, 0);
}

void Kap_DriverConfigPage::slotSettings()
{
	loadPPD();
        if (ppd_)
        {
		Kap_PrinterConfig	dlg(ppd_, this);
		dlg.setUseConflict(true);
                dlg.exec();
        }
}

Kap_Main::e_PageType Kap_DriverConfigPage::getNextPage(QString&, bool)
{
	//return Kap_Main::NameSelector;
	return Kap_Main::BannerSelector;
}
Kap_Main::e_PageType Kap_DriverConfigPage::getType() const
{
	return Kap_Main::DriverConfig;
}

bool Kap_DriverConfigPage::canNextPage(QString& msg)
{
	KBuffer	*buffer = loadPPDAsBuffer();
        if (!buffer)
        {
		msg = i18n("Unable to load driver %s!").arg(ppdFileName_);
                return false;
        }
        KPrinterDescription::get()->setBuffer(buffer);
	return true;
}

void Kap_DriverConfigPage::loadPPD()
{
        KPrinterDescription	*d = KPrinterDescription::get();
        QString	filename = d->getPPD();
        if (filename.isEmpty())
        {
        	Kap_ModelItem	*value = d->getPrinterValues();
                if (!value)
                {
                	QMessageBox::critical(this, i18n("Error"), i18n("Driver not set for printer %1").arg(d->getModel()), QMessageBox::Ok|QMessageBox::Default, 0);
                        if (ppd_)
                        {
                        	globalPpdClose(ppd_);
                                ppd_ = 0;
                        }
                        return;
                }
                filename = value->getPpdFile();
        }
        if (!ppd_ || filename != ppdFileName_)
        {
		if (ppd_)
                {
                	globalPpdClose(ppd_);
	                ppd_ = 0;
                        ppdFileName_ = "";
		}
                ppd_ = globalPpdOpenFile(filename.data());
                if (ppd_)
                {
                	ppdFileName_ = filename;
	                globalPpdMarkDefaults(ppd_);
                }
        }
}

KBuffer* Kap_DriverConfigPage::loadPPDAsBuffer()
{
	loadPPD();
        if (!ppd_) return 0;
        else
        {
	        QString	tmpfile = cupsTempFile(NULL, 0);
	        writeGlobalPpdFile(ppd_, tmpfile.data(), ppdFileName_.data());
	        KBuffer	*buffer = new KBuffer;
	        buffer->load(tmpfile);
	        unlink(tmpfile.data());
                return buffer;
        }
}

//------------------------------------------------------------------------------------------------------

/*void dump(global_ppd_file_t *ppd)
{
	debug("********************************************************");
	debug("Manufacturer: %s",ppd->ppd->manufacturer);
	debug("Model: %s",ppd->ppd->modelname);
	debug("Driver: %s",ppd->ppd->nickname);
        for (int i=0; i<ppd->ppd->num_groups;i++)
        {
        	ppd_group_t	*gr = ppd->ppd->groups+i;
                debug("\nGroup: %s",gr->text);
                for (int k=0;k<gr->num_options;k++)
                {
                	ppd_option_t	*opt = gr->options+k;
                        debug("\nOption: %s (%s)",opt->text, opt->keyword);
                        for (int n=0;n<opt->num_choices;n++)
                        {
				ppd_choice_t	*ch = opt->choices+n;
                                debug("\t%s (%s) %s",ch->text,ch->choice,(ch->marked ? "*" : ""));
                        }
                }
        }
	debug("********************************************************");
}
*/
