

#include "kap_ippprinterselector.h"
#include "kap_ipputil.h"
#include "kprinterdescription.h"
#include "kups.h"

#include <qmessagebox.h>
#include <kapp.h>

Kap_IppPrinterSelector::Kap_IppPrinterSelector(QWidget *parent, const char *name)
	: Kap_PrinterSelector(parent,name)
{
	setTitle(i18n("Choose a printer"));
}

Kap_IppPrinterSelector::~Kap_IppPrinterSelector()
{
}

Kap_Main::e_PageType Kap_IppPrinterSelector::getType() const
{
	return Kap_Main::IppPrinterSelector;
}

Kap_Main::e_PageType Kap_IppPrinterSelector::getNextPage(QString&, bool fake)
{
	if (fake) return Kap_Main::ModelSelector;

	KPrinterDescription	*descr = KPrinterDescription::get();
	descr->setLpdQueue(QString(getPrinter()));
	return Kap_Main::ModelSelector;
}

bool Kap_IppPrinterSelector::canNextPage(QString& msg)
{
	if (!getPrinter())
	{
		msg = i18n("Select a printer !");
		return false;
	}
	return true;
}

bool Kap_IppPrinterSelector::initialize()
{
	KPrinterDescription	*descr = KPrinterDescription::get();
	QStrList	*list = Kap_IppUtil::getPrinterList(descr->getSocketPrinter().data(),descr->getSocketPort(),descr->getUser(),descr->getPassword());
	clearPrinters();
	if (!list)
	{
		QMessageBox::critical(this,i18n("Error"),i18n("Unable to retrieve printer list !"),QMessageBox::Ok|QMessageBox::Default,0);
		return false;
	}
	if (list->count() == 0)
	{
		QMessageBox::warning(this,i18n("Warning"),i18n("No printer found !"),QMessageBox::Ok|QMessageBox::Default,0);
		return true;
	}
	QStrListIterator	it(*list);
	for (;it.current();++it)
		addPrinter(it.current());
	// need to delete object (it is allocated in Kap_IppUtil class)
	delete list;
	return true;
}
