#include "kap_lpdutil.h"

#include <sys/types.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <netdb.h>
#include <stdio.h>
#include <unistd.h>

#include <qstring.h>

QStrList	Kap_LpdUtil::printerlist_;
QString		Kap_LpdUtil::host_;

bool Kap_LpdUtil::hasLpdDaemon(const char *host)
{
	int	sock = openConnection(host);
	bool	val(sock >= 0);
	closeConnection(sock);
	return val;
}

bool Kap_LpdUtil::isValidQueue(const char *host, const char *queue)
{
#ifdef DEBUG
debug("host: %s",host);
#endif
	int	sock = openConnection(host);
	if (sock < 0) return false;
	QString	res;
	res.sprintf("%c%s\n",(char)4,queue);
#ifdef DEBUG
debug("write: %s",res.data());
#endif
	if (::write(sock,res.data(),res.length()) != (int)res.length())
	{
#ifdef DEBUG
debug("connection closed");
#endif
		closeConnection(sock);
		return false;
	}
	res.truncate(0);
	char	buf[1024];
	int	n;
#ifdef DEBUG
debug("reading");
#endif
	while ((n=::read(sock,buf,1023)) > 0)
	{
		buf[n] = 0;
#ifdef DEBUG
debug("%s",buf);
#endif
		res.append(buf);
	}
	closeConnection(sock);
	if (res.isEmpty() || res.find("unknown printer") != -1) return false;
	else return true;
}

const QStrList* Kap_LpdUtil::getPrinterList(const char *host, bool rescan)
{
	// don't do the work if it has already been done
	if (host_ == host && !rescan) return &printerlist_;
	printerlist_.setAutoDelete(true);
	printerlist_.clear();
	QString	cmd;
	cmd.sprintf("rsh %s lpstat -a | awk '{print $1}'",host);
	char	buf[1024];
	FILE	*out = popen(cmd.data(),"r");
	while (fgets(buf,1024,out) != NULL)
	{
#ifdef DEBUG
debug("%s",buf);
#endif
		QString	str(buf);
		printerlist_.append(str.stripWhiteSpace().data());
	}
	pclose(out);
	host_ = host;
	return &printerlist_;
}

int Kap_LpdUtil::openConnection(const char *rhost)
{
	int	sock;
//	int	port = IPPORT_RESERVED - 1;
	struct hostent	*host;
	struct servent	*serv;
	struct sockaddr_in	sin;

	if (!rhost) return -1;
	host = gethostbyname(rhost);
	if (!host) return -1;
	serv = getservbyname("printer","tcp");
	if (!serv) return -1;
	bzero((char*)&sin,sizeof(sin));
	if (host->h_length > (int)sizeof(sin.sin_addr))
		host->h_length = sizeof(sin.sin_addr);
	bcopy(host->h_addr,(caddr_t)&sin.sin_addr,host->h_length);
	sin.sin_family = host->h_addrtype;
	sin.sin_port = serv->s_port;

	sock = socket(AF_INET,SOCK_STREAM,0);
//	sock = rresvport(&port);
	if (sock < 0) return -1;
	// host connecting
	if (::connect(sock,(struct sockaddr*)(&sin),sizeof(sin)) < 0) return -1;
	return sock;
}

void Kap_LpdUtil::closeConnection(int sock)
{
	::close(sock);
}
