/***************************************************************************
                          kap_manumodeldico.cpp  -  description
                             -------------------
    begin                : Sun Feb 13 2000
    copyright            : (C) 2000 by jec
    email                : cuendet@linkvest.ch
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include <cups/ppd.h>


#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/wait.h>



#include <qlabel.h>
#include <qdict.h>
#include <qstrlist.h>
#include <qdir.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qregexp.h>
#include <qmsgbox.h>
#include <qstring.h>


#include <ksimpleconfig.h>
#include <kconfigdata.h>
#include <kapp.h>
#include <kstddirs.h>


#include "kups.h"
#include "kprintermanager.h"
#include "kbuffer.h"

#include "kap_manumodeldico.h"
#include "kap_dbcreator.h"
#include "kups_infos.h"


//------------------------------------------------------------------

bool checkDate(const QFileInfo& dir_, const QDateTime& date_)
{ // returns true if anything in dir_ is newer than date_
	if (dir_.lastModified() > date_) return true;
	else
	{
		QDir	dir(dir_.absFilePath());
		// check most recent file in dir
		const QFileInfoList	*list = dir.entryInfoList(QDir::Files,QDir::Time);
		if (list->count() > 0 && list->getFirst()->lastModified() > date_) return true;
		// check all subdirs
		list = dir.entryInfoList(QDir::Dirs,QDir::Time);
		QFileInfoListIterator	it(*list);
		for (;it.current();++it)
			if (it.current()->fileName() != "." && it.current()->fileName() != "..")
				if (checkDate(*(it.current()), date_)) return true;
		return false;
	}
}

//------------------------------------------------------------------

Kap_ModelItem::Kap_ModelItem()
{
}
Kap_ModelItem::~Kap_ModelItem()
{
}

const QString& Kap_ModelItem::getManufacturer() const
{
	return manufacturer_;
}
void Kap_ModelItem::setManufacturer(const QString& str)
{
	manufacturer_ = str;
	cleanUp();
}

const QString& Kap_ModelItem::getModel() const
{
	return model_;
}
void Kap_ModelItem::setModel(const QString& str)
{
	model_ = str;
        if (shortNickName_.isEmpty()) shortNickName_ = str;
	cleanUp();
}

const QString& Kap_ModelItem::getNickName() const
{
	return nickName_;
}
void Kap_ModelItem::setNickName(const QString& str)
{
	nickName_ = str;
}

const QString& Kap_ModelItem::getShortNickName() const
{
	if (!shortNickName_.isEmpty()) return shortNickName_;
	else return model_;
}
void Kap_ModelItem::setShortNickName(const QString& str)
{
	shortNickName_ = str;
	cleanUp();
}

const QString& Kap_ModelItem::getPpdFile() const
{
	return ppdFile_;
}
void Kap_ModelItem::setPpdFile(const QString& str)
{
	ppdFile_ = str;
}

const QString& Kap_ModelItem::getStatus() const
{
	return status_;
}
void Kap_ModelItem::setStatus(const QString& str)
{
	status_ = str;
}

const QString& Kap_ModelItem::getPnpManufacturer() const
{
	return pnpManufacturer_;
}
void Kap_ModelItem::setPnpManufacturer(const QString& str)
{
	pnpManufacturer_ = str;
}
	
const QString& Kap_ModelItem::getPnpModel() const
{
	return pnpModel_;
}
void Kap_ModelItem::setPnpModel(const QString& str)
{
	pnpModel_ = str;
}

const QString& Kap_ModelItem::getPnpCmd() const
{
	return pnpCmd_;
}
void Kap_ModelItem::setPnpCmd(const QString& str)
{
	pnpCmd_ = str;
}

const QString& Kap_ModelItem::getPnpDescr() const
{
	return pnpDescr_;
}
void Kap_ModelItem::setPnpDescr(const QString& str)
{
	pnpDescr_ = str;
}

void Kap_ModelItem::cleanUp()
{
	if (!manufacturer_.isEmpty())
        {
		if (!model_.isEmpty()) model_ = cleanModel(manufacturer_, model_);
		if (!shortNickName_.isEmpty()) shortNickName_ = cleanModel(manufacturer_, shortNickName_);
        }
}

//-------------------------------------------------------------------------------------
QString cleanModel(const QString& manu_, const QString& model_)
{
	QString	str = model_.stripWhiteSpace();
        if (str.find(manu_, 0, false) == 0 && str[manu_.length()].isSpace()) str = str.right(str.length()-manu_.length()).stripWhiteSpace();
        return str;
}


//-------------------------------------------------------------------------------------
Kap_ManuModelDico* Kap_ManuModelDico::unique_ = 0;
Kap_ManuModelDico* Kap_ManuModelDico::get(bool check)
{
	if (unique_ == 0)
	{
		unique_ = new Kap_ManuModelDico();
		if (!unique_->initialize())
		{
			delete unique_;
			unique_ = 0;
		}
	}
	else if (check) unique_->checkDatabase();
	return unique_;
}



//-------------------------------------------------------------------------------------
Kap_ManuModelDico::Kap_ManuModelDico()
: manuDico_(17, false), pnpManuDico_(17, false)
{
	manuDico_.setAutoDelete(true);
	pnpManuDico_.setAutoDelete(true);
	manuList_.setAutoDelete(true);
}
Kap_ManuModelDico::~Kap_ManuModelDico()
{
}

//-------------------------------------------------------------------------------------
bool Kap_ManuModelDico::makePrintersDb(const QString& ppdDirName, const QString& dbFilename, bool showDlg)
{
	Kap_DBCreator	*dlg = new Kap_DBCreator(kapp->mainWidget());
	bool		value(true);
	dlg->setPpdDirectory(ppdDirName);
	dlg->setDbFilename(dbFilename);
	if (!dlg->exec())
	{
		KPrinterManager::setLastError(dlg->errorString());
		value = false;
	}
	delete dlg;
	return value;
}

//-------------------------------------------------------------------------------------
bool Kap_ManuModelDico::initialize()
{
	return checkDatabase(true);
}

bool Kap_ManuModelDico::checkDatabase(bool firstTime)
{
	QFileInfo	dbDir(Kups_Infos::get()->getDriversDir());
	QFileInfo	dbFile(KGlobal::dirs()->localkdedir() + "/share/apps/kups/printers_db.txt");
	bool		needToLoad(false);

	if (!dbFile.exists() || checkDate(dbDir,dbFile.lastModified()))
	{
		if (!makePrintersDb(dbDir.absFilePath(), dbFile.absFilePath(), true))
		{
			return false;
		}
		needToLoad = true;
	}
	if (needToLoad || firstTime) return loadDatabase(dbFile.absFilePath());
	return true;
}

bool Kap_ManuModelDico::loadDatabase(const QString& dbFilename)
{
	QFile file(dbFilename);
	if (!file.exists() || !file.open(IO_ReadOnly))
	{
		return false;
	}
	
	QTextStream stream(&file);
	
	Kap_ModelItem* modelItem = 0;
	manuDico_.clear();
	pnpManuDico_.clear();
	manuList_.clear();
	
	bool eof = stream.eof();
	QString line = stream.readLine();
	while ( !eof )
	{
		QString descr;
		QString value;
		if (splitString(line, descr, value))
		{
			// PPD file name
			if (descr == "*ppdFileName:")
			{
				addNewItem(modelItem);
				// It's a new item
				modelItem = new Kap_ModelItem;

				modelItem->setPpdFile(value);
			}

			// Other infos
			if (descr == "*Manufacturer:" && modelItem != 0)
			{
				// to avoid any problem, all manufacturer's names are made upper case
				modelItem->setManufacturer(value.upper());
			}
			if (descr == "*ModelName:" && modelItem != 0)
			{
				modelItem->setModel(value);
				if (modelItem->getShortNickName().isEmpty()) modelItem->setShortNickName(value);
			}
			if (descr == "*ShortNickName:" && modelItem != 0)
			{
				modelItem->setShortNickName(value);
			}
			if (descr == "*NickName:" && modelItem != 0)
			{
				modelItem->setNickName(value);
			}

			// PPD status
			if (descr == "*ppdStatus:" && modelItem != 0)
			{
				modelItem->setStatus(value);
			}
			
			// PNP infos
			if (descr == "*pnpManufacturer:" && modelItem != 0)
			{
				modelItem->setPnpManufacturer(value);
			}
			if (descr == "*pnpModel:" && modelItem != 0)
			{
				modelItem->setPnpModel(value);
			}
			if (descr == "*pnpCmd:" && modelItem != 0)
			{
				modelItem->setPnpCmd(value);
			}
			if (descr == "*pnpDescr:" && modelItem != 0)
			{
				modelItem->setPnpDescr(value);
			}
			
		}
#ifdef DEBUG
		else
		{
debug("Line '%s' could not be parsed!\n", line.data() );
		}
#endif
	
		eof = stream.eof();
		line = stream.readLine();

	}

	addNewItem(modelItem);

	return true;
}

bool Kap_ManuModelDico::addNewItem(Kap_ModelItem* modelItem)
{
	if ( modelItem == 0 )
	{
		return false;
	}

	// check is non-empty model (if model is not empty, then nickname will too).
	if (modelItem->getModel().isEmpty())
	{
#ifdef DEBUG
debug("PPD %s is skipped because *ModelName: is not set\n", modelItem->getPpdFile().data() );
#endif
		delete modelItem;
		return false;
	}
	if (modelItem->getManufacturer().isEmpty())
	{
		// Try to guess the manufacturer in nick name
		const QString& str = modelItem->getNickName();
		int index = str.find(' ', 0);
		QString manu = str.mid(0, index);
		modelItem->setManufacturer(manu.upper());
		if (modelItem->getManufacturer().isEmpty())
		{
#ifdef DEBUG
debug("PPD %s is skipped because *Manufacturer: is not set\n", modelItem->getPpdFile().data() );
#endif
			delete modelItem;
			return false;
		}
#ifdef DEBUG
		else
		{
debug("PPD %s had no *Manufacturer. Took '%s' in *ModelName instead\n", modelItem->getPpdFile().data(), manu.data() );
		}
#endif
	}

	// Insert in normal dict (manufacturer and model are supposed non empty).
	QDict<Kap_ModelItemList>* modelDict = manuDico_.find( modelItem->getManufacturer() );
	if (modelDict == 0)
	{
		modelDict = new QDict<Kap_ModelItemList>(17, false);
		modelDict->setAutoDelete(true);
		manuList_.append( modelItem->getManufacturer().data() );
		manuDico_.insert( modelItem->getManufacturer(), modelDict );
	}
#warning Nickname
	Kap_ModelItemList	*driverList = modelDict->find( modelItem->getShortNickName() );
	if (driverList == 0)
	{
		driverList = new Kap_ModelItemList();
		driverList->setAutoDelete(true);
#warning Nickname
		modelDict->insert( modelItem->getShortNickName(), driverList );
	}
	driverList->append(modelItem);

	// Insert in PNP dict only if informations available
	if (!modelItem->getPnpManufacturer().isEmpty() && !modelItem->getPnpModel().isEmpty())
	{
		modelDict = pnpManuDico_.find( modelItem->getPnpManufacturer() );
		if (modelDict == 0)
		{
			modelDict = new QDict<Kap_ModelItemList>(17, false);
			modelDict->setAutoDelete(true);
			pnpManuDico_.insert( modelItem->getPnpManufacturer(), modelDict );
		}
		driverList = modelDict->find( modelItem->getPnpModel() );
		if (driverList == 0)
		{
			driverList = new Kap_ModelItemList();
			driverList->setAutoDelete(false);
			modelDict->insert( modelItem->getPnpModel(), driverList );
		}
		driverList->append(modelItem);
	}

	return true;
}

//-------------------------------------------------------------------------------------
bool Kap_ManuModelDico::splitString(const QString& li, QString& descr, QString& value)
{
	QString line = li.simplifyWhiteSpace();
	int index = line.find(':');
	
	descr = line.left(index+1);
	descr = descr.simplifyWhiteSpace();
	descr.replace(QRegExp("^\""), "");
	descr.replace(QRegExp("\"$"), "");
	
	value = line.mid(index+1, line.length()); // All the rest
	value = value.simplifyWhiteSpace();
	value.replace(QRegExp("^\""), "");
	value.replace(QRegExp("\"$"), "");

	return true;
}

//-------------------------------------------------------------------------------------
QDict<Kap_ModelItemList>* Kap_ManuModelDico::getModelList(const QString& manu)
{
	QDict<Kap_ModelItemList>* list = manuDico_.find( manu.data() );
	return list;
}
//-------------------------------------------------------------------------------------
QStrList& Kap_ManuModelDico::getManuList()
{
	return manuList_;
}
	
//-------------------------------------------------------------------------------------
Kap_ModelItemList* Kap_ManuModelDico::getDescriptionForPnp(const QString& manu, const QString& model)
{
	QDict<Kap_ModelItemList>	*modelDict = pnpManuDico_.find( manu );
	if (modelDict)
	{
		Kap_ModelItemList	*list = modelDict->find( model );
		if (list) return list;
	}
	return 0;
}
//-------------------------------------------------------------------------------------
Kap_ModelItemList* Kap_ManuModelDico::getDescription(const QString& manu, const QString& model)
{ // model is indeed "ShortNickName" as drivers are now classified with it.
	QDict<Kap_ModelItemList>	*modelDict = manuDico_.find( manu );
	if (modelDict)
	{
		Kap_ModelItemList	*list = modelDict->find( model );
		if (list) return list;
	}
	return 0;
}

//-------------------------------------------------------------------------------------
KBuffer* Kap_ManuModelDico::getValidPpdFileAsBuffer(const Kap_ModelItem& item, KBuffer& buffer)
{
	QString fileName = item.getPpdFile();
	if (fileName.isEmpty())
	{
		KPrinterManager::setLastError( i18n("PPD file is empty!") );
		return 0;
	}
	else if (fileName[0] != '/') // Not absolute PATH
	{
		fileName = QString("/usr/share/cups/model/") + item.getPpdFile();
	}

	if (!buffer.load(fileName))
	{
		KPrinterManager::setLastError( i18n("Error while loading Valid PPD file!") );
		return 0;
	}
	return &buffer;
}

