/***************************************************************************
                          kap_modelselectorpage.cpp  -  description
                             -------------------
    begin                : Thu Feb 3 2000
    copyright            : (C) 2000 by jec
    email                : cuendet@linkvest.ch
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include <qdir.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qgroupbox.h>
#include <qlistbox.h>
#include <qpushbutton.h>
#include <qmessagebox.h>
#include <qcheckbox.h>
#include <qstrlist.h>

#include <cups/ppd.h>
#include <qtcups/globalppd.h>


#include <kapp.h>
#include <qmessagebox.h>
#include <ksimpleconfig.h>
#include <kfiledialog.h>
#include <klocale.h>


#include "kprintermanager.h"
#include "kap_modelselectorpage.h"
#include "kap_manumodeldico.h"
#include "kprinterdescription.h"
#include "kbuffer.h"
#include "kap_driverselector.h"
#include "kap_printerconfig.h"
#include "kups_infos.h"

Kap_ModelSelectorPage::Kap_ModelSelectorPage(QWidget *parent, const char *name )
:Kap_WizardPage(parent,name)
{
	setTitle(i18n("Printer model"));

	explLabel_ = new QLabel(this);
	explLabel_->setText(i18n("Choose now the manufacturer and the model of your printer"));
	explLabel_->setFixedHeight(explLabel_->sizeHint().height());

	groupBox_ = new QGroupBox(this);

	QHBoxLayout	*groupLayout_ = new QHBoxLayout(groupBox_,10);

	manuBox_ = new QListBox(groupBox_);
	modelBox_ = new QListBox(groupBox_);

	groupLayout_->addWidget(manuBox_, 1);
	groupLayout_->addWidget(modelBox_, 2);

	connect(manuBox_, SIGNAL(highlighted(int)), this, SLOT(slotManuChanged(int)) );
	connect(modelBox_, SIGNAL(highlighted(int)), this, SLOT(slotModelChanged(int)) );

	browsePpd_ = new QPushButton(i18n("Have driver..."),this);
	browsePpd_->setFixedSize(browsePpd_->sizeHint());
	connect(browsePpd_,SIGNAL(clicked()),SLOT(slotBrowseClicked()));

	raw_ = new QCheckBox(i18n("Raw printer (no driver needed)"), this);
	raw_->setFixedHeight(raw_->sizeHint().height());
	connect(raw_, SIGNAL(toggled(bool)), SLOT(slotRaw(bool)));

	genericps_ = new QCheckBox(i18n("Generic Postscript Printer"), this);
	genericps_->setFixedHeight(browsePpd_->sizeHint().height());
	connect(genericps_, SIGNAL(toggled(bool)), SLOT(slotGenericPS(bool)));

	QVBoxLayout	*main_ = new QVBoxLayout(this, 5, 10);
	QHBoxLayout	*sub_ = new QHBoxLayout(0, 0, 10);
	main_->addWidget(explLabel_);
	main_->addWidget(groupBox_, 1);
	main_->addLayout(sub_);
	main_->addWidget(raw_);
	sub_->addWidget(genericps_, 1);
	sub_->addWidget(browsePpd_);
}
Kap_ModelSelectorPage::~Kap_ModelSelectorPage()
{
}

//------------------------------------------------------------------------------------
bool Kap_ModelSelectorPage::initialize()
{
        if (manuBox_->count() == 0)	// not initialized
        {
	        Kap_ManuModelDico* dico = Kap_ManuModelDico::get(true);
		if (dico == 0)
		{
			QMessageBox::critical((isVisible() ? this : kapp->mainWidget()),i18n("Add printer wizard"),KPrinterManager::getLastError().data(),QMessageBox::Ok|QMessageBox::Default,0);
			return false;
		}

		driver_ = "";

		QStrList& manuList = dico->getManuList();
		if (manuList.count() == 0)
		{
			QMessageBox::warning(this,i18n("Add printer wizard"),i18n("No manufacturers defined !"),QMessageBox::Ok|QMessageBox::Default,0);
			return false;
		}
		manuBox_->clear();
		for (int i=0;i<(int)manuList.count();i++)
		{
			manuBox_->insertItem(manuList.at(i));
		}
		manuBox_->sort();
		manuBox_->setCurrentItem(0);

		// Look for generic PS driver
		Kap_ModelItemList	*values = dico->getDescription("POSTSCRIPT","Generic postscript printer");
		genericps_->setEnabled((values && values->count() >= 1u));
        }

	KPrinterDescription	*descr = KPrinterDescription::get();
#warning Nickname
	QString	currmanu = descr->getManufacturer(), currmodel = descr->getShortNickName();
	if (!currmanu.isEmpty() && !currmodel.isEmpty())
	{ // initial values
		QListBoxItem	*item = manuBox_->findItem(currmanu);
		if (item) manuBox_->setCurrentItem(item);
		item = modelBox_->findItem(currmodel);
		if (item) modelBox_->setCurrentItem(item);
	}

	return true;
}

//------------------------------------------------------------------------------------	
void Kap_ModelSelectorPage::slotModelChanged(int index)
{
	model_ = 	modelBox_->text(index);
}

//------------------------------------------------------------------------------------
void Kap_ModelSelectorPage::slotManuChanged(int index)
{
	manu_ = manuBox_->text(index);
	if (!driver_.isEmpty()) return;
	Kap_ManuModelDico* dico = Kap_ManuModelDico::get();
	if (dico == 0)
	{
		QMessageBox::warning(this,i18n("Add printer wizard"),i18n("Can't open drivers file !"),QMessageBox::Ok|QMessageBox::Default,0);
		return;
	}
	QDict<Kap_ModelItemList>* modelList = dico->getModelList(manu_);
	if (modelList == 0 || modelList->count() == 0)
	{
		QMessageBox::warning(this,i18n("Add printer wizard"),KPrinterManager::getLastError(),QMessageBox::Ok|QMessageBox::Default,0);
		return;
	}
	modelBox_->clear();
	QDictIterator<Kap_ModelItemList>	it(*modelList);
	QStrIList	list(true);
	for (;it.current();++it)
		list.inSort( it.currentKey().data() );
	modelBox_->insertStrList(&list);
	modelBox_->setCurrentItem(0);
	model_ = modelBox_->text(0);

	return;
}

//----------------------------------------------------------------------------------------
Kap_Main::e_PageType Kap_ModelSelectorPage::getNextPage(QString&, bool)
{
        if (raw_->isChecked()) return Kap_Main::NameSelector;
        else if (driver_.isEmpty() && KPrinterDescription::get()->getPrinterValues() == 0)
        	return Kap_Main::DriverSelector;
        else return Kap_Main::DriverConfig;
}

Kap_Main::e_PageType Kap_ModelSelectorPage::getType() const
{
	return Kap_Main::ModelSelector;
}

bool Kap_ModelSelectorPage::canNextPage(QString& msg)
{
	KPrinterDescription* parDet = KPrinterDescription::get();

	// first check if raw printer
	if (raw_->isChecked())
	{
		parDet->setManufacturer("");
		parDet->setModel("");
		parDet->setBuffer(0);
		parDet->setPPD("");
		parDet->setDescription("Raw printer");
		parDet->setPrinterValues(0);
                parDet->setNickName("");
		return true;
	}

	if (modelBox_->currentItem() < 0)
	{
		msg = i18n("Select a printer's model");
		return false;
	}
	parDet->setManufacturer(manu_);
	parDet->setModel(model_);
	parDet->setShortNickName(model_);
	if (!driver_.isEmpty())
	{
		parDet->setBuffer(0);
		parDet->setPPD(driver_);
		parDet->setDescription("");
		parDet->setPrinterValues(0);
                parDet->setNickName(nickname_);
	}
	else
	{
		Kap_ManuModelDico* dico = Kap_ManuModelDico::get();
		if (dico == 0)
		{
			msg = i18n("Can't open drivers database !");
			return false;
		}
		Kap_ModelItemList* values = dico->getDescription(manu_, model_);
		if (!values || values->count() == 0)
		{
			msg = i18n("Unable to find a driver file for this model !");
			return false;
		}
/*		Kap_ModelItem	*value = values->first();
		if (values->count() > 1) value = Kap_DriverSelector::selectDriver(values, this, false);
		if (!value)
		{
			return false;
		}

		parDet->setPrinterValues( value );
		parDet->setDescription( value->getPnpDescr() );
                parDet->setNickName( value->getNickName() );

		KBuffer	*buffer = new KBuffer;
		if (dico->getValidPpdFileAsBuffer(*value,*buffer) != 0)
		{
			parDet->setBuffer(buffer);
		}
		else
		{
			msg = i18n("Unable to find a driver file for this model !");
			return false;
		}*/
		if (values->count() == 1)
                {
                	Kap_ModelItem	*value = values->first();
			parDet->setPrinterValues( value );
			parDet->setDescription( value->getPnpDescr() );
	                parDet->setNickName( value->getNickName() );

			/*KBuffer	*buffer = new KBuffer;
			if (dico->getValidPpdFileAsBuffer(*value,*buffer) != 0)
			{
				parDet->setBuffer(buffer);
			}
			else
			{
				msg = i18n("Unable to find a driver file for this model !");
				return false;
			}*/
                }
                else
                {
                	parDet->setPrinterValues(0);
                        parDet->setDescription("");
                        parDet->setNickName("");
                        parDet->setBuffer(0);
                }
	}

	if (parDet->getDescription().isEmpty())
	{
		QString	descr = manu_ + " / " + model_;
		parDet->setDescription(descr);
	}

	return true;
}

//----------------------------------------------------------------------------------------
void Kap_ModelSelectorPage::slotBrowseClicked()
{
	driver_ = KFileDialog::getOpenFileName(Kups_Infos::get()->getDriversDir(),"*.ppd *.ppd.gz|Driver file (PPD)",this,i18n("Select a driver"));
	if (!driver_.isEmpty())
	{ // driver selected, load it and update widgets
		global_ppd_file_t	*ppd = globalPpdOpenFile(driver_.data());
		if (ppd)
		{
			manuBox_->clear();
			modelBox_->clear();
			manuBox_->insertItem(ppd->ppd->manufacturer != 0 ? QString(ppd->ppd->manufacturer).upper() : i18n("(Unknown)"));
#warning NickName
                        modelBox_->insertItem(ppd->ppd->shortnickname != 0 ? QString(ppd->ppd->shortnickname) : i18n("(Unknown)"));
                        nickname_ = ppd->ppd->nickname;
			manuBox_->setCurrentItem(0);
			modelBox_->setCurrentItem(0);
			globalPpdClose(ppd);
		}
		else
		{
			QMessageBox::critical(this,i18n("Error"),i18n("Not a valid driver !"),QMessageBox::Ok|QMessageBox::Default,0);
			driver_ = "";
		}
	}
}

//----------------------------------------------------------------------------------------
void Kap_ModelSelectorPage::slotRaw(bool on)
{
	if (on)
	{
		manuBox_->clearSelection();
		modelBox_->clearSelection();
	}
	else
	{
		manuBox_->setSelected(manuBox_->findItem(manu_), true);
		modelBox_->setSelected(modelBox_->findItem(model_), true);
	}
	manuBox_->setEnabled(!on);
	modelBox_->setEnabled(!on);
	browsePpd_->setEnabled(!on);
	genericps_->setEnabled(!on);
}

//----------------------------------------------------------------------------------------
void Kap_ModelSelectorPage::slotGenericPS(bool on)
{
	if (on)
	{
		manuBox_->setCurrentItem(manuBox_->findItem("POSTSCRIPT"));
	}
	manuBox_->setEnabled(!on);
	modelBox_->setEnabled(!on);
	raw_->setEnabled(!on);
	browsePpd_->setEnabled(!on);
}
