#include "kap_smbutil.h"

#include <stdio.h>
#include <string.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/time.h>
#include <unistd.h>

//-------------------------------------------------------------------------------

#define NMBHDRSIZE 13

struct nmbhdr {
	unsigned short id; /* name_trn_id */

	unsigned char R:1; /* response (BOOL) */
	unsigned char opcode:4; /* opcode - could be switched with above?
*/
	unsigned char AA:1; /* broadcast (BOOL) */
	unsigned char TC:1; /* truncate (BOOL) */
	unsigned char RD:1; /* recursion desired */
	unsigned char RA:1; /* recursion available */
	unsigned char unless:2;
	unsigned char B:1;
	unsigned char RCODE:4;

	unsigned short que_num;
	unsigned short rep_num;
	unsigned short num_rr;
	unsigned short num_rrsup;
	unsigned char namelen;
};


struct typez {
    unsigned short type;
    unsigned short type2;
};

//-------------------------------------------------------------------------------

Kap_SmbUtil*	Kap_SmbUtil::util_ = 0;

Kap_SmbUtil* Kap_SmbUtil::get()
{
	if (!util_) util_ = new Kap_SmbUtil;
	return util_;
}

void Kap_SmbUtil::release()
{
	if (util_)
	{
		delete util_;
		util_ = 0;
	}
}

Kap_SmbUtil::Kap_SmbUtil()
{
	smbinfos_.setAutoDelete(true);
}

Kap_SmbUtil::~Kap_SmbUtil()
{
}

bool Kap_SmbUtil::getIPList(QStrList& IPList)
{
	QString	cmd("nmblookup \\* | grep 00 | sed 's/*<00>//'");
	FILE	*out = popen(cmd,"r");
	char	buffer[1024];
	while (fgets(buffer,1024,out) != NULL)
	{
		char	*c = strchr(buffer,'\n');
		if (!c) continue;
		else
		{
			*c = '\0';
			IPList.append(buffer);
		}
	}
	return true;
}

bool Kap_SmbUtil::getNMBInfo(unsigned long ip, QString& workgroup, QString& server)
{
	struct sockaddr_in sin_dst;
	struct nmbhdr *nmb;
	char *data;
	struct typez *typz;
	char buffer[1024];
	int socket_client;
	int longueur = sizeof(struct sockaddr_in);
	int count;
	struct _nameinfo {
		char name[15];
		char type;
		char res;
		char fill;
	} *ninf;
	struct timeval  timeout = {0, 100000};
	fd_set rfds;
	bool canread = true;


	bzero(buffer, sizeof(buffer));
	nmb = (struct nmbhdr *) buffer;
	data = (char *) (buffer + NMBHDRSIZE);
	typz = (struct typez *) (buffer + NMBHDRSIZE + 33);

	sin_dst.sin_family = AF_INET;
	sin_dst.sin_port = htons(137);
	sin_dst.sin_addr.s_addr = ip;

	memcpy(data, "CKAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA", 34);

	nmb->id = 0x600;
	nmb->R = 0;			/* 0 for question 1 for response */
	nmb->opcode = 0;		/* 0 = query */
	nmb->que_num = htons(1);	/* i have only 1 question :) */
	nmb->namelen = 0x20;
	typz->type = 0x2100;
	typz->type2 = 0x0100;

	socket_client = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);

	sendto(socket_client, buffer, 50, 0, (struct sockaddr *) &sin_dst, longueur);
	FD_ZERO(&rfds);
	FD_SET(socket_client,&rfds);
	if (select(socket_client+1,&rfds,0,0,&timeout) < 0 || !FD_ISSET(socket_client,&rfds))
		// there nothing to read, timeout reached.
		canread = false;

	bzero(buffer, sizeof(buffer));
	nmb = (struct nmbhdr *) (buffer);

	if (canread)
	{
		if (recvfrom(socket_client, buffer, sizeof(buffer), 0, (struct sockaddr *) &sin_dst, (socklen_t*)&longueur) != -1)
		{
			if (nmb->rep_num != 0)
			{
				/*set to first nameblock after header */
				count = 0;
				ninf = (struct _nameinfo *) (buffer + 57 + (count++ * sizeof(struct _nameinfo)));
				while (server.isEmpty() && (57 + (count * sizeof(struct _nameinfo)) < 1024))
				{
					if (ninf->type == 0)
					{
						QString	tmp( QCString(ninf->name, 15) );
						server = tmp.data();
					}
					ninf = (struct _nameinfo *) (buffer + 57 + (count++ * sizeof(struct _nameinfo)));
				}
				if (server.isEmpty())
				{
					close(socket_client);
					return false;
				}
				while (workgroup.isEmpty() && (57 + (count * sizeof(struct _nameinfo)) < 1024))
				{
					if (ninf->type == 0)
					{
						QString	tmp( QCString(ninf->name, 15) );
						workgroup = tmp.data();
					}
					ninf = (struct _nameinfo *) (buffer + 57 + (count++ * sizeof(struct _nameinfo)));
				}
			}
		}
	}
	close(socket_client);
	return canread;
}

const QList<SmbInfo>& Kap_SmbUtil::getSmbList(bool scan)
{
	if (scan)
	{
		smbinfos_.clear();
		QStrList	IPList;
		IPList.setAutoDelete(true);
		if (getIPList(IPList))
		{
			QStrListIterator	it(IPList);
			for (;it.current();++it)
			{
				unsigned long	ip = inet_addr(it.current());
				SmbInfo	*info = new SmbInfo;
				info->PrinterList.setAutoDelete(true);
				info->IP = it.current();
				if (getNMBInfo(ip,info->Workgroup,info->Server))
				{
					smbinfos_.append(info);
#ifdef DEBUG
debug("%s : %s %s",it.current(),info->Workgroup.data(),info->Server.data());
#endif
				}
				else
				{
#ifdef DEBUG
debug("%s : unable to retrieve info",it.current());
#endif
					delete info;
				}
			}
		}
	}
	return smbinfos_;
}

bool Kap_SmbUtil::getPrinters(SmbInfo* smb, const QString& user, const QString& passwd)
{
	QString	cmd;
	if (!user.isEmpty())
		cmd.sprintf("smbclient -L %s -W %s -U %s%%%s -I %s ",smb->Server.stripWhiteSpace().data(),smb->Workgroup.stripWhiteSpace().data(),user.data(),passwd.data(),smb->IP.data());
	else
		cmd.sprintf("smbclient -L %s -W %s -N -I %s ",smb->Server.stripWhiteSpace().data(),smb->Workgroup.stripWhiteSpace().data(),smb->IP.data());
	cmd.append(" | awk '{print $1,$2}' | grep Printer | awk '{print $1}'");
#ifdef DEBUG
debug("%s",cmd.data());
#endif
	FILE	*out = popen(cmd.data(),"r");
	char	buffer[1024];
	smb->PrinterList.clear();
	while (fgets(buffer,1024,out) != NULL)
	{
		char	*c = strchr(buffer,'\n');
		if (c)
		{
			*c = '\0';
#ifdef DEBUG
debug("found printer %s",buffer);
#endif
			smb->PrinterList.append(buffer);
		}
	}
	return true;
}

