#include "kpp_driverpage.h"

#include <qlabel.h>
#include <qlayout.h>
#include <qpushbutton.h>
#include <klocale.h>

//#include <cups/ppd.h>

#include "kprinterdescription.h"
#include "kaddprinterwizard.h"
#include "kbuffer.h"

Kpp_DriverPage::Kpp_DriverPage(QWidget *parent, const char *name)
	: QWidget(parent,name)
{
	desc_ = 0;
	modified_ = false;
	initWidget();
}

Kpp_DriverPage::~Kpp_DriverPage()
{
}

void Kpp_DriverPage::initWidget()
{
	infoLabel_ = new QLabel(this);
	infoLabel_->setAlignment(Qt::AlignTop|Qt::AlignLeft|Qt::ExpandTabs);

	QPushButton	*change_ = new QPushButton(i18n("Change..."), this);
	change_->setFixedSize(change_->sizeHint());
	connect(change_,SIGNAL(clicked()),SLOT(changeClicked()));

	QVBoxLayout	*mainLayout_ = new QVBoxLayout(this, 10);
	mainLayout_->addWidget(infoLabel_, 1);
	mainLayout_->addWidget(change_, 0, Qt::AlignLeft|Qt::AlignVCenter);
}

void Kpp_DriverPage::changeClicked()
{
	// set initial values in static structure
        KPrinterDescription	*std = KPrinterDescription::get();
	std->setManufacturer(desc_->getManufacturer());
	std->setModel(desc_->getModel());
	std->setNickName(desc_->getNickName());
	std->setShortNickName(desc_->getShortNickName());
	std->setURI(desc_->getDeviceUri());
	std->setType(desc_->getType());

	Kap_AddPrinterWizard	wizard(this);
	wizard.setWizardControl(Kap_Main::ModelSelector, Kap_Main::DriverConfig, true);
	if (wizard.exec())
	{
		// update printer description
		desc_->setPPD(std->getPPD());
		desc_->setManufacturer(std->getManufacturer());
		desc_->setModel(std->getModel());
		if (std->getBuffer())
		{
			KBuffer	*newBuf = new KBuffer(*(std->getBuffer()));
			desc_->setBuffer(newBuf);
		}
		desc_->setPrinterValues(std->getPrinterValues());

		modified_ = true;

		// update view
		setInfos(desc_);
	}
}

void Kpp_DriverPage::collectInfos(KPrinterDescription *d)
{
	d->setPPD(desc_->getPPD());
	d->setManufacturer(desc_->getManufacturer());
	d->setModel(desc_->getModel());
	if (desc_->getBuffer())
	{
		KBuffer	*newBuf = new KBuffer(*(desc_->getBuffer()));
		d->setBuffer(newBuf);
	}
}

void Kpp_DriverPage::setInfos(KPrinterDescription *d)
{
	desc_ = d;
	if (!d->getModel().isEmpty())
	{
		QString	str;
		str.append(i18n("Manufacturer: %1\n").arg(d->getManufacturer()));
		str.append(i18n("Model: %1").arg(d->getModel()));
		infoLabel_->setText(str);
	}
	else
	{
		infoLabel_->setText(i18n("No driver found"));
	}
	infoLabel_->setMinimumHeight(infoLabel_->sizeHint().height());
}
