/***************************************************************************
                          kprinter.cpp  -  description
                             -------------------
    begin                : Thu Oct 28 1999
    copyright            : (C) 1999 by jec
    email                : cuendet@linkvest.ch
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <klocale.h>
#include <cups/ipp.h>

#include "kprinter.h"
#include "kprintermanager.h"
#include "kprinterdescription.h"

#include "kjob.h"
#include "kclasshelper.h"
#include "kprinterhelper.h"
#include "kbuffer.h"


KPrinter::KPrinter()
:name_(),
description_(),
owner_(),
state_(Idle),
isDefault_(false),
location_(),
info_(),
modelinfo_(),
localURI_(),
ippURIs_(),
isColor_(false),
supportedMedia_(),
defaultMedia_(),
printerType_(-1),
isAcceptingJobs_(true),
defaultCopies_(1),
copiesRangeMin_(1),
copiesRangeMax_(1),
pageRangeSupported_(false),
defaultOrientation_(Portrait)
{
}
KPrinter::KPrinter(const KPrinter& other)
:KObject(other)
{
	//scopy(other);
	copy(&other);
}
KPrinter::~KPrinter()
{
}

void KPrinter::copy(const KPrinter *other)
{
	name_ = other->name_;
	owner_ = other->owner_;
	state_ = other->state_;
	isDefault_ = other->isDefault_;
	location_ = other->location_;
	info_ = other->info_;
	description_ = other->description_;
	modelinfo_ = other->modelinfo_;
	localURI_ = other->localURI_;
	ippURIs_ = other->ippURIs_;
	isColor_ = other->isColor_;
	supportedMedia_ = other->supportedMedia_;
	defaultMedia_ = other->defaultMedia_;
	printerType_ = other->printerType_;
	isAcceptingJobs_ = other->isAcceptingJobs_;
	defaultCopies_ = other->defaultCopies_;
	copiesRangeMin_ = other->copiesRangeMin_;
	copiesRangeMax_ = other->copiesRangeMax_;
	pageRangeSupported_ = other->pageRangeSupported_;
	defaultOrientation_ = other->defaultOrientation_;

	mediaList_ = other->mediaList_;
	orientationList_ = other->orientationList_;

	supportedBanners_ = other->supportedBanners_;
	startBanner_ = other->startBanner_;
	endBanner_ = other->endBanner_;
}

int KPrinter::getNbJobs() const
{
	return jobs_.count();
}
KJob* KPrinter::getJobAt(int i)
{
	return jobs_.at(i);
}

void KPrinter::addJobToList(KJob* job)
{
	jobs_.append(job);
}

void KPrinter::removeJobFromList(KJob* job, bool remove)
{
	// take the kob out of job list (without deleting it) and
	// delete it if "remove" is true.
	if (jobs_.find(job) != -1) jobs_.take();
	if (remove) delete job;
}

KJob* KPrinter::getJobByID(int id)
{
	for (int i=0;i<getNbJobs();i++)
	{
		KJob* job = getJobAt(i);
		if (job->getID() == id)
		{
			return job;
		}
	}
	return 0;
}

int KPrinter::getNbColumns()
{
	return 8;
}

QString KPrinter::getStaticColumnNameAt(int i)
{
/*	KStubPrinter p;
	return p.getColumnNameAt(i);*/
	switch (i)
	{
	   case 0: return i18n("Name");
	   case 1: return i18n("Nb jobs");
	   case 2: return i18n("Uri");
	   case 3: return i18n("Owner");
	   case 4: return i18n("State");
	   case 5: return i18n("Orientation");
	   case 6: return i18n("Location");
	   case 7: return i18n("State reason");
	   case 8: return i18n("<Unknown>");
	}
  return i18n("<Unknown>");
}

QString KPrinter::getColumnNameAt(int i)
{
	return getColumnNameOrValueAt(i, true);
}
	
QString KPrinter::getColumnNameOrValueAt(int i, bool name)
{
	switch(i)
	{
	case 0: // name
		if (name)
		{
			return i18n("Name");
		}
		else
		{
			return name_;
		}
	case 1: // Nb jobs
		if (name)
		{
			return i18n("Nb jobs");	
		}
		else
		{
			//QString nbJobStr_;
			nbJobStr_.setNum(getNbJobs());
			return nbJobStr_;
		}
	case 2: // local uri
		if (name)
		{
			return i18n("uri");	
		}
		else
		{
			return localURI_;
		}
	case 3: // Owner
		if (name)
		{
			return i18n("Owner");	
		}
		else
		{
			return owner_;
		}
	case 4: // State
		if (name)
		{
			return i18n("State");	
		}
		else
		{
			switch (state_)
			{
			case Idle:
				stateStr_ = i18n("Idle");
				break;
			case Processing:
				stateStr_ = i18n("Processing");
				break;
			case Stopped:
				stateStr_ = i18n("Stopped");
				break;
			};
			return stateStr_;
		}
	case 5: // Orientation
		if (name)
		{
			return i18n("Orientation");	
		}
		else
		{
			switch(defaultOrientation_)
			{
			case Landscape:
				orientStr_ = "Landscape";
				break;
			case Portrait:
				orientStr_= "Portrait";
				break;
			case Rev_Landscape:
				orientStr_ = "Reverse Landscape";
				break;
			case Rev_Portrait:
				orientStr_ = "Reverse portrait";
				break;
			};
			return orientStr_;
		}
	case 6: // Location
		if (name)
		{
			return i18n("Location");	
		}
		else
		{
			return location_;
		}
	case 7: // State reason
		if (name)
		{
			return i18n("State reason");
		}
		else
		{
			return stateReason_;
		}
	default:
		break;
	};

	static QString colName;
	colName = i18n("<Unknown>");
	return colName;
}

QString KPrinter::getColumnValueAt(int i)
{
	return getColumnNameOrValueAt(i, false);
}

void KPrinter::addMedia(const QString& str)
{
	mediaList_.append(str);
}

void KPrinter::addOrientation(Orientation orient)
{
	orientationList_.append( orient );
}

void KPrinter::fillDescription(KPrinterDescription *)
{
}

bool KPrinter::isClass() const
{
	return false;
}





//------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------
KRealPrinter::KRealPrinter()
: KPrinter()
{
	helper_ = KPrinterHelper::get();
}

KRealPrinter::KRealPrinter(const KRealPrinter& other)
: KPrinter(other)
{
	KRealPrinter::copy(&other);
	helper_ = KPrinterHelper::get();
}

KRealPrinter::~KRealPrinter()
{
}

bool KRealPrinter::sendCommand(Command command)
{
	return helper_->sendCommand(this, command);
}

bool KRealPrinter::sendCommand(Command command, KPrinter *other)
{
	return helper_->sendCommand(this, command, other);
}

bool KRealPrinter::printFile(const QString& page)
{
	return helper_->printFile(this, page);
}

bool KRealPrinter::printBuffer(const KBuffer& page)
{
	return helper_->printBuffer(this, page);
}

void KRealPrinter::setAttribute(ipp_attribute_t* attr)
{
	helper_->setAttribute(this, attr);
}

bool KRealPrinter::isRemote() const
{
	return (getPrinterType() & CUPS_PRINTER_REMOTE);
}

bool KRealPrinter::isImplicit() const
{
	return (getPrinterType() & CUPS_PRINTER_IMPLICIT);
}

void KRealPrinter::fillDescription(KPrinterDescription *d)
{
	// general
	d->setName(getName());
	d->setLocation(getLocation());
	d->setDescription(getDescription());
	// backend
	if (!isRemote() && !getLocalURI().isEmpty())
	{
		QString	uri = getLocalURI();
		d->setURI(uri);
		int	p = uri.find(':');
		QString	prot = uri.left(p);
		if (prot == "parallel")
		{
			d->setType(KPrinterManager::e_Parallel);
			d->setParportNumber(uri.right(1).toInt());
		}
		else if (prot == "serial")
		{
			d->setType(KPrinterManager::e_Serial);
		}
		else if (prot == "usb")
		{
			d->setType(KPrinterManager::e_USB);
			d->setParportNumber(uri.right(1).toInt());
		}
		else if (prot == "lpd")
		{
			d->setType(KPrinterManager::e_LPD);
			p += 3;
			int	p2 = uri.find('/',p);
			d->setLpdServer(uri.mid(p,p2-p));
			p = p2+1;
			p2 = uri.find('/',p);
			if (p2 == -1) d->setLpdQueue(uri.right(uri.length()-p));
			else d->setLpdQueue(uri.mid(p,p2-p));
		}
		else if (prot == "socket")
		{
			d->setType(KPrinterManager::e_Socket);
			QString	host;
			int	port;
			p += 3;
			int	p2 = uri.find(':',p);
			if (p2 != -1)
			{
				host = uri.mid(p,p2-p);
				p = p2+1;
				p2 = uri.find('/',p);
				if (p2 == -1) port = uri.right(uri.length()-p).toInt();
				else port = uri.mid(p,p2-p).toInt();
			}
			else
			{
				port = 9100;
				p2 = uri.find('/',p);
				if (p2 == -1) host = uri.right(uri.length()-p);
				else host = uri.mid(p,p2-p);
			}
			d->setSocketInfo(host,port);
		}
		else if (prot == "ipp" || prot == "http")
		{
			d->setType(KPrinterManager::e_IPP);
			p += 3;
			QString	host, pr;
			int	port;
			int	p2 = uri.find(':',p);
			if (p2 != -1)
			{
				host = uri.mid(p,p2-p);
				p = p2+1;
				p2 = uri.find('/',p);
				port = uri.mid(p,p2-p).toInt();
				p = p2+1;
			}
			else
			{
				p2 = uri.find('/',p);
				host = uri.mid(p,p2-p);
				p = p2+1;
				port = 631;
			}
			p2 = uri.find('/',p);
			if (p2 == -1) pr = uri.right(uri.length()-p);
			else pr = uri.mid(p,p2-p);
			d->setSocketInfo(host,port);
			d->setLpdQueue(pr);
		}
		else if (prot == "smb")
		{
			d->setType(KPrinterManager::e_SMB);
			p += 3;
			QString	s1, s2, s3;
			int	p2 = uri.find('/',p);
			s1 = uri.mid(p,p2-p);
			p = p2+1;
			p2 = uri.find('/',p);
			if (p2 != -1)
			{
				s2 = uri.mid(p,p2-p);
				p = p2+1;
				p2 = uri.find('/',p);
				if (p2 != -1) s3 = uri.mid(p,p2-p);
				else if (p < (int)uri.length()) s3 = uri.right(uri.length()-p);
			}
			else if (p < (int)uri.length()) s2 = uri.right(uri.length()-p);
			if (s3.isEmpty()) d->setSMBInfo("",s1,s2);
			else d->setSMBInfo(s1,s2,s3);
		}
		else if (prot == "file")
		{
			d->setType(KPrinterManager::e_File);
		}
	}
	// PPD
	QString	ppdFileName = cupsGetPPD(getName().data());
	ppd_file_t	*ppd = ppdOpenFile(ppdFileName.data());
	if (ppd)
	{
		d->setModel(QString(ppd->modelname));
		d->setManufacturer(QString(ppd->manufacturer));
		d->setNickName(QString(ppd->nickname));
                d->setShortNickName(QString(ppd->shortnickname));
		KBuffer	*buf = new KBuffer();
		buf->load(ppdFileName);
		d->setBuffer(buf);
		d->setPPD(QString(""));
		ppdClose(ppd);
	}
	else
	{
		d->setModel(QString(""));
		d->setManufacturer(QString(""));
		d->setPPD(QString(""));
		d->setBuffer(0);
                d->setNickName(QString(""));
                d->setShortNickName(QString(""));
	}
	unlink(ppdFileName.data());
	// Banners
	d->setSupportedBanners(getSupportedBanners());
	d->setStartBanner(getStartBanner());
	d->setEndBanner(getEndBanner());
}
//------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------



	
	
KClassPrinter::KClassPrinter()
{
	helper_ = KClassHelper::get();
}
KClassPrinter::KClassPrinter(const KClassPrinter& other)
:KPrinter(other)
{
	copy(&other);
	helper_ = KClassHelper::get();
}
KClassPrinter::~KClassPrinter()
{
}

void KClassPrinter::addPrinterToClass(const QString& str)
{
	KPrinter* prn = KPrinterManager::get()->getPrinterByUri(str);
	if (prn != 0)
	{
		addPrinterToClass(prn);
	}
#ifdef DEBUG
	else
	{
debug("Warning : printer %s not found",str.data());
	}
#endif
}

void KClassPrinter::addPrinterToClass(KPrinter* printer)
{
	printersList_.append(printer);
}

int KClassPrinter::getNbPrinters() const
{
	return printersList_.count();
}

KPrinter* KClassPrinter::getPrinterAt(int i)
{
	return printersList_.at(i);
}
const KPrinter* KClassPrinter::getPrinterAt(int i) const
{
	return ((KClassPrinter*)this)->printersList_.at(i);
}

void KClassPrinter::copy(const KClassPrinter *other)
{
	KPrinter::copy(other);
	printersList_.setAutoDelete(false);
	printersList_.clear();
	int nb = other->getNbPrinters();
	for (int i=0;i<nb;i++)
	{
		KRealPrinter* p = (KRealPrinter*)other->getPrinterAt(i);
		addPrinterToClass(p);
	}
}

bool KClassPrinter::sendCommand(Command command, KPrinter* other)
{
  return helper_->sendCommand(this, command, other);
}

bool KClassPrinter::printFile(const QString& page)
{
	return helper_->printFile(this, page);
}

bool KClassPrinter::printBuffer(const KBuffer& page)
{
	return helper_->printBuffer(this, page);
}

bool KClassPrinter::sendCommand(Command command)
{
	return helper_->sendCommand(this, command);
}

void KClassPrinter::setAttribute(ipp_attribute_t* attr)
{
	helper_->setAttribute(this, attr);
}

bool KClassPrinter::isRemote() const
{
	return (getPrinterType() & CUPS_PRINTER_REMOTE);
}

bool KClassPrinter::isClass() const
{
	return true;
}

void KClassPrinter::fillDescription(KPrinterDescription *d)
{
	// general
	d->setName(getName());
	d->setLocation(getLocation());
	d->setDescription(getDescription());
        d->setType(KPrinterManager::e_Class);
	// members
	int	n = getNbPrinters();
	QStrList	*prlist = new QStrList();
	for (int i=0;i<n;i++)
	{
		KPrinter	*pr = getPrinterAt(i);
		prlist->append(pr->getName().data());
	}
	d->setPrintersList(prlist);
}
