/***************************************************************************
                          kprintermanager.h  -  description
                             -------------------
    begin                : Thu Oct 28 1999
    copyright            : (C) 1999 by jec
    email                : cuendet@linkvest.ch
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KPRINTERMANAGER_H
#define KPRINTERMANAGER_H


#include <qlist.h>
#include <qstring.h>

#include <cups/cups.h>
#include <cups/http.h>

/**
  *@author jec
*/


class KPrinter;
class KRealPrinter;
class KClassPrinter;
class KPrinterObserver;
class KJob;
class KPrinterDescription;

class KPrinterManager
{
public:
	enum e_AddType
	{
		e_Invalid,
		e_Class,
		e_Parallel,
		e_Serial,
		e_IPP,
		e_LPD,
		e_SMB,
		e_Socket,
		e_USB,
		e_File
	};

	KPrinterManager();
	~KPrinterManager();

	static KPrinterManager* get();
  static const QString& getLastError();
  static void setLastError(const QString& le);

  static void setUser(const QString& pwd) { user_ = pwd; };
  static const QString& getUser() { return user_; };
  static void setPassword(const QString& pwd) { password_ = pwd; };
  static const QString& getPassword() { return password_; };

	void setAllPrintersDiscarded(bool disc);
	void deleteAllDiscardedPrinters();

	void setAllClassesDiscarded(bool disc);
	void deleteAllDiscardedClasses();

	void setAllJobsDiscarded(bool disc);
	void deleteAllDiscardedJobs();

  void addObserver(KPrinterObserver* observer);

  void notifyObservers();
	bool refresh();
	bool remove(KPrinter* printer);

	bool addPrinter(KPrinterDescription& descr, bool update = true);
	bool modifyPrinter(KPrinter *printer, KPrinterDescription* descr);

  int getNbPrinters();
  KRealPrinter* getPrinterAt(int i);
  KRealPrinter* getPrinterByName(const QString& name);
  KRealPrinter* getPrinterByUri(const QString& uri, bool tryWithName = false);

  int getNbClasses();
  KClassPrinter* getClassAt(int i);
  KClassPrinter* getClassByName(const QString& name);
  KClassPrinter* getClassByUri(const QString& uri, bool tryWithName = false);

  KJob* getJobByID(int id);

  void setDefaultPrinterUri(const QString& str);
  void setDefaultPrinterName(const QString& str);
  void setDefaultPrinter(KPrinter* printer);

protected:
  bool removePrinterToList(KRealPrinter* printer);
	bool addPrinterToList(KRealPrinter* printer);
  bool removeClassToList(KClassPrinter* printer);
	bool addClassToList(KClassPrinter* printer);

  bool initialize();

protected:
	bool addReal(KPrinterDescription& descr);
	bool addClass(KPrinterDescription& descr);

	bool refreshPrinters();
	bool refreshClasses();
	bool refreshDefault();
	bool refreshJobs();

private:

	QList<KPrinterObserver> observersList_;
	bool	notifyOn_;

  QList<KRealPrinter> printers_;
  QList<KClassPrinter> classes_;

	static QString user_;
	static QString password_;
  static KPrinterManager* unique_;
  static QString lastError_;

	http_t* http_;
};

#endif







