/***************************************************************************
                          kups_configdialog.cpp  -  description
                             -------------------
    begin                : Fri Jan 5 2001
    copyright            : (C) 2001 by Michael Goffioul
    email                : goffioul@imec.be
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kups_configdialog.h"

#include <klocale.h>
#include <qpixmap.h>
#include <kiconloader.h>
#include <qvbox.h>
#include <qslider.h>
#include <qlineedit.h>
#include <qcheckbox.h>
#include <qmessagebox.h>

#include "optionstimer.h"
#include "optionsserver.h"
#include "optionsdirspage.h"
#include "optionstestpage.h"
#include "kups_infos.h"

Kups_ConfigDialog::Kups_ConfigDialog(QWidget *parent, const char *name )
: KDialogBase(IconList,i18n("Configuration"),Ok|Cancel,Ok,parent,name,true)
{
	createDialog();

	resize(500,300);
}

Kups_ConfigDialog::~Kups_ConfigDialog(){
}

void Kups_ConfigDialog::createDialog()
{
	QVBox	*box;

	// Server options
	box = addVBoxPage(i18n("Server"), i18n("CUPS server"), DesktopIcon("gear", 32));
	server_ = new OptionsServerPage(box);
	// Dirs options
	box = addVBoxPage(i18n("Folders"), i18n("CUPS location"), DesktopIcon("folder", 32));
	dirs_ = new OptionsDirsPage(box);
	// Test page options
	box = addVBoxPage(i18n("Test page"), i18n("Test page"), DesktopIcon("fileprint", 32));
	test_ = new OptionsTestPage(box);
	// Timer options
	box = addVBoxPage(i18n("Timer"), i18n("Refresh timer"), DesktopIcon("clock", 32));
	timer_ = new OptionsTimerPage(box);
}

void Kups_ConfigDialog::slotOk()
{
	Kups_Infos	*infos = Kups_Infos::get();

	// simple check
	bool	ok;
	int	val = server_->port_->text().toInt(&ok);
	if (!ok || val < 1)
	{
		QMessageBox::critical(this, i18n("Error"), i18n("Wrong port number"), QMessageBox::Ok|QMessageBox::Default, 0);
		goto end;
	}

	// Timer
	infos->setTimerEnabled( timer_->usetimer_->isChecked() );
	infos->setRefreshTime( timer_->timerslider_->value() );

	// Server
	infos->setServer( server_->host_->text() );
	infos->setPort( server_->port_->text().toInt() );
	infos->setUser( server_->login_->text() );
	infos->setPassword( server_->password_->text() );

	// Test page
	infos->setTestPage( test_->testpage_->text() );

	// Dirs
	infos->setStandardCUPS( dirs_->standardinstall_->isChecked() );
	if (infos->isStandardCUPS())
	{
		infos->setCUPSRoot("");
		infos->setDriversDir("/usr/share/cups/model");
		infos->setConfigFile("/etc/cups/cupsd.conf");
	}
	else
	{
		infos->setCUPSRoot( dirs_->rootedit_->text() );
		infos->setConfigFile( dirs_->configedit_->text() );
		infos->setDriversDir( dirs_->driversedit_->text() );
	}

end:
	KDialogBase::slotOk();
}

void Kups_ConfigDialog::initDialog()
{
	Kups_Infos	*infos = Kups_Infos::get();

	// Timer
	timer_->timerslider_->setValue(infos->getRefreshTime());
	timer_->usetimer_->setChecked(infos->isTimerEnabled());

	// Server
	server_->host_->setText(infos->getServer());
	QString	num;
	num.setNum(infos->getPort());
	server_->port_->setText(num);
	server_->login_->setText(infos->getUser());
	server_->password_->setText(infos->getPassword());

	// Test page
	test_->testpage_->setText(infos->getTestPage());

	// Dirs
	dirs_->rootedit_->setText(infos->getCUPSRoot());
	dirs_->driversedit_->setText(infos->getDriversDir());
	dirs_->configedit_->setText(infos->getConfigFile());
	dirs_->standardinstall_->setChecked(infos->isStandardCUPS());
}
