/***************************************************************************
                          kups_helper.cpp  -  description
                             -------------------
    begin                : Mon Nov 29 1999
    copyright            : (C) 1999 by jec
    email                : cuendet@linkvest.ch
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include <cups/cups.h>
#include <klocale.h>

#include "kups_helper.h"
#include "kups_ipprequest.h"
#include "kups_infos.h"

#include "kprinter.h"
#include "kprintermanager.h"



Kups_Helper::Kups_Helper()
{
}
Kups_Helper::~Kups_Helper()
{
}

bool Kups_Helper::getErrorString(Kups_IppRequest* req, KPrinter* p, QString& str)
{
	return getErrorString(req->getResponseStatus(), p, str);
}
bool Kups_Helper::getErrorString(ipp_status_t status, KPrinter* printer, QString& str)
{
	bool ret = false;
	switch(status)
	{
	case IPP_OK:
		str = "";
		ret = true;
		break;
  case IPP_OK_SUBST:
  	str = "OK but subst ???";
		ret = true;
		break;
  case IPP_OK_CONFLICT:
  	str = "OK but conflict ???";
		ret = true;
		break;
  case IPP_BAD_REQUEST:
  	str = "Bad request";
		ret = true;
		break;
  case IPP_FORBIDDEN:
  	str = "Operation forbidden";
  	ret = true;
		break;
  case IPP_NOT_AUTHENTICATED:
  	str = "Not authenticated. Perhaps wrong password";
		ret = true;
		break;
  case IPP_NOT_AUTHORIZED:
  	str = "Not authorized";
		ret = true;
		break;
  case IPP_NOT_POSSIBLE:
  	str = "Not possible";
		ret = true;
		break;
  case IPP_TIMEOUT:
  	str = "Request timeout";
		ret = true;
		break;
  case IPP_NOT_FOUND:
  	str = "Not found";
		ret = true;
		break;
  case IPP_GONE:
  	str = "Gone...";
		ret = true;
		break;
  case IPP_REQUEST_ENTITY:
  	str = "Request entity ???";
		ret = true;
		break;
  case IPP_REQUEST_VALUE:
  	str = "Request value ???";
		ret = true;
		break;
  case IPP_DOCUMENT_FORMAT:
  	str = "Document format";
		ret = true;
		break;
  case IPP_ATTRIBUTES:
  	str = "Attributes ???";
		ret = true;
		break;
  case IPP_URI_SCHEME:
  	str = "URI sheme";
		ret = true;
		break;
  case IPP_CHARSET:
  	str = "Charset";
		ret = true;
		break;
  case IPP_CONFLICT:
  	str = "Conflict";
		ret = true;
		break;
  case IPP_INTERNAL_ERROR:
  	str = "Internal error";
		ret = true;
		break;
  case IPP_OPERATION_NOT_SUPPORTED:
  	str = "Operation not supported";
		ret = true;
		break;
  case IPP_SERVICE_UNAVAILABLE:
  	str = "Serice unavailable";
		ret = true;
		break;
  case IPP_VERSION_NOT_SUPPORTED:
  	str = "Version not supported";
		ret = true;
		break;
		/*
		// Not available since cups-1.1b3
  case IPP_DEVICE_UNAVAILABLE:
  	str.sprintf("Device %s unavailable", printer->getName().data());
		ret = true;
		break;
		*/
/*  case IPP_DEVICE_ERROR:
  	str.sprintf("Device error on %s", printer->getName().data());
		ret = true;
		break;*/
  case IPP_TEMPORARY_ERROR:
  	str = "Temporary error";
		ret = true;
		break;
  case IPP_NOT_ACCEPTING:
  	str = "Not accepting";
		ret = true;
		break;
  case IPP_PRINTER_BUSY:
  	str.sprintf("Printer %s busy", printer->getName().data());
		ret = true;
		break;
	default:
  	str = "";
		break;
	}
	return ret;
}

bool Kups_Helper::sendCommand(KPrinter* printer, KPrinter::Command command, KPrinter* other)
{
	bool ret = false;
	switch(command)
	{
	case KPrinter::RemoveFromClass:
		ret = removeFromClass(printer, other);
		break;
	case KPrinter::AddToClass:
		ret = addToClass(printer, other);
		break;
	default:
		break;
	};
	return ret;
}
bool Kups_Helper::sendCommand(KPrinter* printer, KPrinter::Command command)
{
	bool ret = false;
	switch(command)
	{
	case KPrinter::AsDefault:
		ret = setAsDefault(printer);
		break;
	case KPrinter::Enable:
		ret = setState(printer, CUPS_ACCEPT_JOBS);
		ret = ret && setState(printer, IPP_RESUME_PRINTER);
		break;
	case KPrinter::Disable:
		ret = setState(printer, CUPS_REJECT_JOBS);
		ret = ret && setState(printer, IPP_PAUSE_PRINTER);
		break;
	case KPrinter::Remove:
		ret = remove(printer);
		break;
	default:
	  {
	  	QString str;
	  	const char* name = printer->getName().data();
	  	str.sprintf("The command (%d) sent to printer %s is unknown", command, name);
			KPrinterManager::setLastError(str);
		}
		ret = false;
		break;
	};
	return ret;
}

//---------------------------------------------------------------------
bool Kups_Helper::printBuffer(KPrinter* printer, const KBuffer& pageBuffer)
{
  Kups_IppRequest request;

  request.setOperation(IPP_PRINT_JOB);
  request.setUri("printer-uri", printer->getIppURI());

  request.setMime("document-format", "application/postscript");
  request.setName("requesting-user-name", Kups_Infos::get()->getUser());
  request.setName("job-name", "<unknown>"); // FIXME

  bool ret = false;
  QString res;
  res.sprintf("/printers/%s", printer->getName().data());
	if (!request.doBufferRequest(res, &pageBuffer))
	{
    //KPrinterManager::setLastError("Unable to set default destination!\n");
    ret = false;
	}
  else
  {
	  ipp_status_t status = request.getResponseStatus();
    if (status == IPP_OK)
    {
    	ret = true;
    }
    else if (status == IPP_NOT_FOUND)
    {
    	QString error;
	    error.sprintf("Destination %s does not exist!\n", printer->getName().data());
      KPrinterManager::setLastError(error);
	    ret = false;
    }
    else
    {
    	QString error;
    	error.sprintf("Can't print : returned error code %d", status);
      KPrinterManager::setLastError(error);
      ret = false;
    }
  }
  return ret;
}

//---------------------------------------------------------------------
bool Kups_Helper::printFile(KPrinter* printer, const QString& page)
{
  Kups_IppRequest request;

  request.setOperation(IPP_PRINT_JOB);
  request.setUri("printer-uri", printer->getIppURI());

  request.setMime("document-format", "application/vnd.cups-postscript");
  request.setName("requesting-user-name", Kups_Infos::get()->getUser());
  request.setName("job-name", page);

  bool ret = false;
  QString res;
  res.sprintf("/printers/%s", printer->getName().data());
	if (!request.doFileRequest(res, page))
	{
    //KPrinterManager::setLastError("Unable to set default destination!\n");
    ret = false;
	}
  else
  {
	  ipp_status_t status = request.getResponseStatus();
    if (status == IPP_OK)
    {
    	ret = true;
    }
    else if (status == IPP_NOT_FOUND)
    {
    	QString error;
	    error.sprintf("Destination %s does not exist!\n", printer->getName().data());
      KPrinterManager::setLastError(error);
	    ret = false;
    }
    else
    {
    	QString error;
    	error.sprintf("Can't print : returned error code %d", status);
      KPrinterManager::setLastError(error);
      ret = false;
    }
  }
  bool z = false; // for compiler error
  return ret;
}

//---------------------------------------------------------------------
bool Kups_Helper::setState(KPrinter* printer, ipp_op_t oper)
{
  bool ret = true;

  Kups_IppRequest request;

  request.setOperation(oper);
  request.setUri("printer-uri", printer->getIppURI());


 /*
  * Do the request and get back a response...
  */

  if (!request.doRequest("/admin/"))
  {
    //KPrinterManager::setLastError("Unable to set default destination!\n");
    ret = false;
  }
  else
  {
    if (request.getResponseStatus() == IPP_NOT_FOUND)
    {
    	QString error;
	    error.sprintf("Destination %s does not exist!\n", printer->getName().data());
      KPrinterManager::setLastError(error);
	    ret = false;
    }
  }
  return ret;
}

//---------------------------------------------------------------------
bool Kups_Helper::setAsDefault(KPrinter* printer)
{
  bool ret = true;

  Kups_IppRequest request;

  request.setOperation(CUPS_SET_DEFAULT);
  request.setUri("printer-uri", printer->getIppURI());


 /*
  * Do the request and get back a response...
  */

  if (!request.doRequest("/admin/"))
  {
    //KPrinterManager::setLastError("Unable to set default destination!\n");
    ret = false;
  }
  else
  {
    if (request.getResponseStatus() == IPP_NOT_FOUND)
    {
    	QString error;
	    error.sprintf("Destination %s does not exist!\n", printer->getName().data());
      KPrinterManager::setLastError(error);
	    ret = false;
    }
  }
  return ret;
}

//-----------------------------------------------------------------------
bool Kups_Helper::removeFromClass(KPrinter* printer, KPrinter* other)
{
	QString pUri = printer->getIppURI();
	QString cUri = other->getIppURI();
	QStringList	uris;

	Kups_IppRequest req;
	req.setOperation( IPP_GET_PRINTER_ATTRIBUTES );
	req.setUri("printer-uri", cUri);
 	if (!req.doRequest("/classes/"))
 	{
 		KPrinterManager::setLastError(i18n("Class does not exist !"));
 		return false;
 	}
 	ipp_attribute_t	*members = req.findAttribute("member-uris",IPP_TAG_URI);
 	ipp_attribute_t	*names = req.findAttribute("member-names",IPP_TAG_NAME);
 	bool	found(false);
 	if (members && names)
 		for (int i=0;i<members->num_values;i++)
 			if (printer->getName() != names->values[i].string.text)
 				uris += QString(members->values[i].string.text);
 			else found = true;
 	if (!found)
 	{
 		KPrinterManager::setLastError(i18n("Unable to find \"%1\" in \"%2\" !").arg(printer->getName()).arg(other->getName()));
 		return false;
 	}

	Kups_IppRequest request;
	request.setOperation( CUPS_ADD_CLASS );
	request.setUri("printer-uri", cUri);
	request.setUris("member-uris",uris);

	return request.doRequest("/admin/");
}

//-----------------------------------------------------------------------
bool Kups_Helper::addToClass(KPrinter* printer, KPrinter* other)
{
	QString pUri = printer->getIppURI();
	QString cUri = other->getIppURI();
	QStringList	uris;
	
	Kups_IppRequest req;
	req.setOperation( IPP_GET_PRINTER_ATTRIBUTES );
	req.setUri("printer-uri", cUri);
 	if (!req.doRequest("/classes/"))
 	{
 		KPrinterManager::setLastError(i18n("Class does not exist !"));
 		return false;
 	}
 	ipp_attribute_t	*members = req.findAttribute("member-uris",IPP_TAG_URI);
 	if (members)
 		for (int i=0;i<members->num_values;i++)
 			uris += QString(members->values[i].string.text);
 	uris += pUri;

	Kups_IppRequest request;
	request.setOperation( CUPS_ADD_CLASS );
	request.setUri("printer-uri", cUri);
	request.setUris("member-uris",uris);

	return request.doRequest("/admin/");
}

//-----------------------------------------------------------------------
bool Kups_Helper::remove(KPrinter* printer)
{
	KPrinterManager::setLastError("Kups_Helper::remove() not implemented");
	return false;
}

//-----------------------------------------------------------------------
bool Kups_Helper::setAttribute(KPrinter* printer, ipp_attribute_t* attr)
{
	if (attr->group_tag != IPP_TAG_PRINTER)
 	{
 		return false;
 	}
 	
 	
	if ( strcmp(attr->name, "printer-name" ) == 0 )
	{
		printer->setName(QString(attr->values[0].string.text));
	}
	else if ( strcmp(attr->name, "printer-location" ) == 0 )
	{
		printer->setLocation(QString(attr->values[0].string.text));
	}
	else if ( strcmp(attr->name, "printer-info" ) == 0 )
	{
		printer->setDescription(QString(attr->values[0].string.text));
	}
	else if ( strcmp(attr->name, "printer-make-and-model" ) == 0 )
	{
		printer->setModelInfo(QString(attr->values[0].string.text));
	}
	else if ( strcmp(attr->name , "printer-state") == 0 )
	{
		KPrinter::State state = KPrinter::Idle;
		switch(attr->values[0].integer)
	  {
	  case IPP_PRINTER_IDLE:
			state	= KPrinter::Idle;
		  break;
	  case IPP_PRINTER_PROCESSING:
			state	= KPrinter::Processing;
		  break;
		case IPP_PRINTER_STOPPED:
			state	= KPrinter::Stopped;
			break;
		default:
   		sleep(0);
		  break;
	  };
	  printer->setState(state);
  }
  else if ( strcmp(attr->name, "printer-state-message") == 0 )
  {
  	printer->setStateReason(QString(attr->values[0].string.text));
  }
  else if ( strcmp(attr->name, "device-uri") == 0 )
  {
  	printer->setLocalURI(QString(attr->values[0].string.text));
  }
  else if ( strcmp(attr->name, "color-supported") == 0 )
  {
  	printer->setIsColor(attr->values[0].integer);
	}
	else if ( strcmp(attr->name, "media-supported") == 0 )
  {
  	for (int i=0;i<attr->num_values;i++)
   	{
	  	printer->addMedia(QString(attr->values[i].string.text));
	  }
  }
  else if ( strcmp(attr->name, "media-default") == 0 )
  {
		printer->setDefaultMedia(QString(attr->values[0].string.text));
  }
  else if ( strcmp(attr->name, "job-sheets-supported") == 0 )
  {
	for (int i=0;i<attr->num_values;i++)
		printer->addBanner(QString(attr->values[i].string.text));
  }
  else if ( strcmp(attr->name, "job-sheets-default") == 0 )
  {
	if (attr->num_values > 0) printer->setStartBanner(QString(attr->values[0].string.text));
	if (attr->num_values > 1) printer->setEndBanner(QString(attr->values[1].string.text));
  }
  else if ( strcmp(attr->name, "printer-type") == 0 )
  {
		printer->setPrinterType(attr->values[0].integer);
	}
  else if ( strcmp(attr->name, "printer-is-accepting-jobs") == 0 )
  {
  	printer->setIsAcceptingJobs(attr->values[0].integer);
  }
  else if ( strcmp(attr->name, "printer-uri") == 0 )
  {
  	printer->addIppURI(QString(attr->values[0].string.text));
  }
  else if ( strcmp(attr->name, "printer-uri-supported") == 0 )
  {
  	printer->addIppURI(QString(attr->values[0].string.text));
  }
  else if ( strcmp(attr->name, "copies-supported") == 0 )
  {
  	printer->setCopiesRange(attr->values[0].range.lower, attr->values[0].range.upper);
  }
  else if ( strcmp(attr->name, "copies-default") == 0 )
  {
  	printer->setDefaultCopies(attr->values[0].integer);
  }
  else if ( strcmp(attr->name, "page-ranges-supported") == 0 )
  {
  	printer->setPageRangeSupported(attr->values[0].boolean);
  }
  else if ( strcmp(attr->name, "orientation-requested-supported") == 0 )
  {
  	for (int i=0;i<attr->num_values;i++)
   	{
	  	int z = attr->values[i].integer;
	   	switch(z)
  	 	{
			case IPP_PORTRAIT:
	   		printer->addOrientation(KPrinter::Portrait);
	   		break;
			case IPP_LANDSCAPE:
	   		printer->addOrientation(KPrinter::Landscape);
	   		break;
			case IPP_REVERSE_LANDSCAPE:
	   		printer->addOrientation(KPrinter::Rev_Landscape);
	   		break;
			case IPP_REVERSE_PORTRAIT:
	   		printer->addOrientation(KPrinter::Rev_Portrait);
	   		break;
   		};
   	}
  }
  else if ( strcmp(attr->name, "orientation-requested-default") == 0 )
  {
  	int z = attr->values[0].integer;
   	switch(z)
 	 	{
		case IPP_PORTRAIT:
   		printer->setDefaultOrientation(KPrinter::Portrait);
   		break;
		case IPP_LANDSCAPE:
   		printer->setDefaultOrientation(KPrinter::Landscape);
   		break;
		case IPP_REVERSE_LANDSCAPE:
   		printer->setDefaultOrientation(KPrinter::Rev_Landscape);
   		break;
		case IPP_REVERSE_PORTRAIT:
   		printer->setDefaultOrientation(KPrinter::Rev_Portrait);
  		break;
  		};
  }
  else
  {
  	return false;
  }

  return true;
}


