/***************************************************************************
                          kups_printeritem.cpp  -  description
                             -------------------
    begin                : Sun Nov 14 1999
    copyright            : (C) 1999 by jec
    email                : cuendet@linkvest.ch
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kups_printeritem.h"

#include <kiconloader.h>
#include <qpainter.h>
#include <qfont.h>
#include <klocale.h>

#include <cups/ipp.h>

#include "kprinter.h"
#include "kinstancemanager.h"


Kups_ListItem::Kups_ListItem(QListView* parent, const char *name)
:QListViewItem(parent, name),
isDiscarded_(false)
{
}
Kups_ListItem::Kups_ListItem(QListViewItem* parent, const char *name)
:QListViewItem(parent, name),
isDiscarded_(false)
{
};
Kups_ListItem::~Kups_ListItem()
{
}




Kups_PrinterItem::Kups_PrinterItem(QListView* parent, bool allColumns, const char *name )
:Kups_ListItem(parent,name),
printer_(0),
allColumns_(allColumns)
{
}
Kups_PrinterItem::Kups_PrinterItem(QListViewItem* parent, bool allColumns, const char *name)
:Kups_ListItem(parent,name),
printer_(0),
allColumns_(allColumns)
{
}
Kups_PrinterItem::~Kups_PrinterItem()
{
}

void Kups_PrinterItem::setPrinter(KPrinter* printer)
{
	printer_ = printer;
}

KPrinter* Kups_PrinterItem::getPrinter()
{
	return printer_;
}

QString Kups_PrinterItem::text(int column) const
{
/*	KPrinter* p = (KPrinter*)printer_;
	if (p != 0)
	{
		QString	value = p->getColumnValueAt(column);
		return value;
	}*/
	return QListViewItem::text(column);
}

Kups_PrinterItem* Kups_PrinterItem::addChild(KPrinter* printer, PrinterType type)
{
	Kups_PrinterItem* child = getChildWithPrinter(printer);
	if (child != 0)
	{
		child->setIsDiscarded(false);
	}
	else
	{
		child = new Kups_PrinterItem(this, allColumns_);
		child->setPrinter(printer);
	}
	if (printer->getName() == KInstanceManager::get()->getPrinterDefaultInstance())
		child->setText( 0, i18n("%1 (default)").arg(printer->getName()) );
	else
		child->setText( 0, printer->getName() );
	child->setPrinterType(type);
	bool	hasClassIcon = (type == CLASS || printer->isImplicit());

	KPrinter* p = child->getPrinter();
	QString	iconPath(hasClassIcon ? "kups_class" : "kups_printer");
	// check default
	if (p->isDefault()) iconPath += "_default";
	// check remote or not
	if (p->isRemote() && !hasClassIcon) iconPath += "_remote";
	// check state
	if (p->getState() == KPrinter::Stopped) iconPath += "_stopped";
	else if (p->getState() == KPrinter::Processing) iconPath += "_process";
	// set icon
	child->setPixmap(0,SmallIcon(iconPath));
	
	return child;
}

void Kups_PrinterItem::setAllChildrenDiscarded(bool disc)
{
	Kups_PrinterItem* child = (Kups_PrinterItem*)firstChild();
	while (child != 0)
	{
		child->setIsDiscarded(disc);
		child = (Kups_PrinterItem*)child->nextSibling();
	}
}

void Kups_PrinterItem::deleteAllChildrenDiscarded()
{
	Kups_PrinterItem* child = (Kups_PrinterItem*)firstChild();
	while (child != 0)
	{
		QListViewItem* c = 0;
		if (child->isDiscarded())
		{
			c = child;
		}
		child = (Kups_PrinterItem*)child->nextSibling();
		/*
		if (c && c->isSelected())
		{
			QListViewItem* temp = c->itemAbove();
			if (temp == 0)
			{
				temp = c->parent();
			}
			if (temp != 0)
			{
				temp->setSelected(true);
			}
		}
		*/
		delete c;
	}
}

Kups_PrinterItem* Kups_PrinterItem::getChildWithPrinter(KPrinter* printer)
{
	Kups_PrinterItem* child = (Kups_PrinterItem*)firstChild();
	while (child != 0)
	{
		if (child->getPrinter() == printer)
		{
			return child;
		}
		child = (Kups_PrinterItem*)child->nextSibling();
	}
	return 0;
}

void Kups_PrinterItem::paintCell(QPainter *p, const QColorGroup& cg, int column, int width, int alignment)
{
	QFont	f(p->font());
	if (column == 0 && printer_ && printer_->isDefault()) f.setItalic(true);
	else f.setItalic(false);
	p->setFont(f);
	QListViewItem::paintCell(p, cg, column, width, alignment);
}
