#include "kups_printerproperties.h"

#include <klocale.h>
#include <qmessagebox.h>

#include <cups/cups.h>

#include "kpp_generalpage.h"
#include "kpp_backendpage.h"
#include "kpp_driverpage.h"
#include "kpp_memberspage.h"
#include "kprinterdescription.h"
#include "kprinter.h"
#include "kprintermanager.h"
#include "kaddprinterwizard.h"

Kups_PrinterProperties::Kups_PrinterProperties(QWidget *parent, const char *name)
	: QTabDialog(parent,name,true)
{
	desc_ = 0;
	printer_ = 0;
	general_ = 0;
	backend_ = 0;
	driver_ = 0;
	members_ = 0;
	initDialog();
}

Kups_PrinterProperties::~Kups_PrinterProperties()
{
	if (desc_) delete desc_;
	KPrinterDescription::release();
}

void Kups_PrinterProperties::done(int res)
{
	if (printer_ && desc_ && res == Accepted)
	{
		KPrinterDescription	d;
		// first collect infos
		general_->collectInfos(&d);
		if (printer_->getPrinterType() & CUPS_PRINTER_CLASS) d.setType(KPrinterManager::e_Class);
		bool	nameChanged = (d.getName() != desc_->getName());
		if (backend_ && (nameChanged || backend_->hasChanged())) backend_->collectInfos(&d);
		if (driver_ && (nameChanged || driver_->hasChanged())) driver_->collectInfos(&d);
		if (members_ && (nameChanged || members_->hasChanged())) members_->collectInfos(&d);
		if (!KPrinterManager::get()->modifyPrinter(printer_,&d))
			QMessageBox::critical(this, i18n("Error"), i18n("%1 edition failed:\n%2").arg(printer_->getPrinterType() & CUPS_PRINTER_CLASS ? i18n("Class") : i18n("Printer")).arg(KPrinterManager::get()->getLastError()), QMessageBox::Ok|QMessageBox::Default, 0);
	}
	QTabDialog::done(res);
}

void Kups_PrinterProperties::initDialog()
{
	// general Page
	general_ = new Kpp_GeneralPage(this);
	addTab(general_,i18n("General"));

	// buttons
	setCancelButton(i18n("Cancel"));
	setOkButton(i18n("OK"));

	resize(300,200);
}

void Kups_PrinterProperties::setPrinter(KPrinter *printer)
{
	if (printer_) return;
	printer_ = printer;
	setCaption(i18n("Properties of %1").arg(printer_->getName()));
	desc_ = new KPrinterDescription();
	printer_->fillDescription(desc_);

	general_->setInfos(desc_);
	if (printer_->getPrinterType() & CUPS_PRINTER_CLASS)
	{
		// members page
		members_ = new Kpp_MembersPage(this);
		addTab(members_,i18n("Members"));

		members_->setInfos(desc_);
	}
	else
	{
		// backend page
		backend_ = new Kpp_BackendPage(this);
		addTab(backend_,i18n("Backend"));
		// driver page
		driver_ = new Kpp_DriverPage(this);
		addTab(driver_,i18n("Driver"));

		backend_->setInfos(desc_);
		driver_->setInfos(desc_);
	}
}
