/***************************************************************************
                          kupsview.cpp  -  description
                             -------------------
    begin                : Mon Nov  8 19:59:21 CET 1999
    copyright            : (C) 1999 by jec
    email                : cuendet@linkvest.ch
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <stdio.h>

// include files for Qt
//#include <qprinter.h>
//#include <qpainter.h>
//#include <qlistview.h>
//#include <qlineedit.h>
//#include <qcombobox.h>
//#include <qlayout.h>
#include <qtimer.h>
#include <qtabwidget.h>
#include <qvaluelist.h>

// KDE includes
#include <qmessagebox.h>
#include <kiconloader.h>
#include <kfiledialog.h>
#include <kstddirs.h>
#include <klocale.h>
#include <kmainwindow.h>
#include <kaction.h>

#include <qtcups/qcupsprinter.h>

// application specific includes
#include "kupsview.h"
#include "kups_printeritem.h"
#include "kups.h"
#include "kups_printerview.h"
#include "kups_jobsview.h"
#include "kups_printerinfo.h"
#include "kups_ipprequest.h"
#include "kups_testpagegenerator.h"
#include "kprinter.h"
#include "kprintermanager.h"
#include "kups_printerpropview.h"
#include "kups_printerinstances.h"

void stopTimer()
{
	KupsView::get()->stopTimer();
}
void startTimer()
{
	KupsView::get()->startTimer();
}
void triggerUpdate()
{
	stopTimer();
	KupsView::get()->slotTimeout();
}

//----------------------------------------------------------------------------
KupsView* KupsView::unique_ = 0;


//----------------------------------------------------------------------------
KupsView::KupsView(QWidget *parent, KConfig* config, const char* name)
:QSplitter(parent, name),
isValid_(false)
{
	if(unique_ != 0)
	{
		QMessageBox::warning(this,"Kups","KupsView is NOT UNIQUE",QMessageBox::Ok|QMessageBox::Default,0);
	}
	unique_ = this;

  //setBackgroundMode(PaletteBase);

  infos_.setServer("localhost");
  infos_.setPort(631);
  infos_.setUser("root");

	config_ = config;
	readOptions(config_);
	
	if (!createWidgets())
	{
		isValid_ = false;
		return;
	}
	if (!initWidgets())
	{
		isValid_ = false;
		return;
	}
	isValid_ = true;
	return;
}

KupsView::~KupsView()
{
	saveOptions(config_);
}

KupsView* KupsView::get()
{
	return unique_;
}

void KupsView::readOptions(KConfig* config)
{
	Kups_Infos* infos = Kups_Infos::get();
	
	config->setGroup("Global");
	
	infos->setUser( config->readEntry("User","root") );
	infos->setServer( config->readEntry("Server","localhost") );
	infos->setPort( config->readNumEntry("Port",631) );
	infos->setConfigFile( config->readEntry("ConfigFile","/etc/cups/cupsd.conf") );
	infos->setTestPage( config->readEntry("TestPage",locate("appdata","testprint.ps")) );
	infos->setRefreshTime( config->readNumEntry("RefreshTime",2) );
	infos->setTimerEnabled( config->readBoolEntry("TimerEnabled",true) );
	infos->setStandardCUPS( config->readBoolEntry("StandardCUPS",true) );
	infos->setDriversDir( config->readEntry("Drivers","/usr/share/cups/model") );
	infos->setCUPSRoot( config->readEntry("CUPSRoot","") );

	// check some values
	if (infos->getPort() < 1)
	{
		infos->setPort(631);
	}
	if (infos->getRefreshTime() > 20 || infos->getRefreshTime() < 1)
	{
		infos->setRefreshTime(2);
	}
}
void KupsView::saveOptions(KConfig* config)
{
	Kups_Infos* infos = Kups_Infos::get();

	config->setGroup("Global");

	config->writeEntry("User", infos->getUser() );
	config->writeEntry("Server", infos->getServer() );
	config->writeEntry("Port", infos->getPort() );
	config->writeEntry("ConfigFile", infos->getConfigFile() );
	config->writeEntry("TestPage", infos->getTestPage() );
	config->writeEntry("RefreshTime", infos->getRefreshTime() );
	config->writeEntry("TimerEnabled", infos->isTimerEnabled() );
	config->writeEntry("StandardCUPS", infos->isStandardCUPS() );
	config->writeEntry("Drivers", infos->getDriversDir() );
	config->writeEntry("CUPSRoot", infos->getCUPSRoot() );
}

bool KupsView::createWidgets()
{
	printerView_ = new Kups_PrinterView(this, "printerView", this, false);
	tabs_ = new QTabWidget(this, "tabs");
	printerInfo_ = new Kups_PrinterInfo(tabs_, "printerInfo");
	jobsView_ = new Kups_JobsView(tabs_, "jobsView");
	propView_ = new Kups_PrinterPropView(tabs_, "propView");
	Kups_PrinterInstances	*instancesView_ = new Kups_PrinterInstances(this, "instancesView");

	QObject::connect(printerView_, SIGNAL(selectionChanged(QListViewItem*)), printerInfo_, SLOT(printerSelected(QListViewItem*)));
	QObject::connect(printerView_, SIGNAL(selectionChanged(QListViewItem*)), jobsView_, SLOT(selectionChanged(QListViewItem*)));
	QObject::connect(printerView_, SIGNAL(selectionChanged(QListViewItem*)), propView_, SLOT(slotPrinterSelected(QListViewItem*)));
	QObject::connect(printerView_, SIGNAL(selectionChanged(QListViewItem*)), instancesView_, SLOT(slotPrinterSelected(QListViewItem*)));
	connect(printerView_, SIGNAL(selectionChanged(QListViewItem*)), SLOT(slotPrinterSelected(QListViewItem*)));

	tabs_->addTab(printerInfo_, SmallIconSet("kups_printer"), i18n("Infos"));
	tabs_->addTab(jobsView_, SmallIconSet("folder"), i18n("Jobs"));
	tabs_->addTab(propView_, SmallIconSet("configure"), i18n("Properties"));
	tabs_->addTab(instancesView_, SmallIconSet("fileprint"), i18n("Instances"));

	timer_ = new QTimer(this);
	connect(timer_, SIGNAL(timeout()), this, SLOT(slotTimeout()));
	startTimer();

	QValueList<int>	sizes;
	sizes.append(150);
	sizes.append(300);
	setSizes(sizes);

	return true;
}

bool KupsView::initWidgets()
{
	return true;
}

void KupsView::slotTimeout()
{
  KupsApp::get()->slotStatusMsg(i18n("Refreshing printers, classes and jobs..."));
	if (!KPrinterManager::get()->refresh())
	{
	  KupsApp::get()->slotStatusMsg(i18n("Can't connect to CUPS server. Check your options!"));
	  printerView_->firstChild()->setPixmap(0, SmallIcon("kups_printer_stopped"));
	}
	else
	{
	  KupsApp::get()->slotStatusMsg(i18n("Refresh done."));
	  printerView_->firstChild()->setPixmap(0, SmallIcon("kups_printer"));
	}
	startTimer();
}

void KupsView::stopTimer()
{
	timer_->stop();
}
void KupsView::startTimer()
{
	if (infos_.isTimerEnabled())
		timer_->start(infos_.getRefreshTime()*1000, true);
}

void KupsView::printPSFile()
{
// print any file on any printer/instance
	QString	filename = KFileDialog::getOpenFileName(QString::null,QString::null,this);
	QCupsPrinter	printer;
	if (!filename.isEmpty() && printer.setup(this))
	{
		printer.printFile(filename.data());
	}
}

void KupsView::print(const QString& printerName)
{
	KPrinter* printer(0);
	if (!printerName.isEmpty()) printer = KPrinterManager::get()->getPrinterByName( printerName );
	else
	{
		Kups_PrinterItem	*item_ = (Kups_PrinterItem*)(printerView_->currentItem());
		if (!item_ || item_->getPrinterType() != Kups_PrinterItem::PRINTER)
			return;	// no printer name given, and no printer printer (we shouldn't be here !!)
		printer = item_->getPrinter();
	}
	if (printer == 0)
	{
		QMessageBox::critical(this,i18n("Print test page"),i18n("Can't find a valid printer for testing !"),QMessageBox::Ok|QMessageBox::Default,0);
		return;
	}

	if (Kups_Infos::get()->getTestPage().isEmpty())
		QMessageBox::critical(this,i18n("Print test page"),i18n("Test page not specified (see Options dialog)"),QMessageBox::Ok|QMessageBox::Default,0);
	else
		Kups_TestPageGenerator::printPage(this, printer);
}

void KupsView::slotPrinterSelected(QListViewItem *item)
{
	bool	on = (item && ((Kups_PrinterItem*)item)->getPrinterType() == Kups_PrinterItem::PRINTER);
	KPrinter	*pr(0);
	if (on) pr = ((Kups_PrinterItem*)item)->getPrinter();
	KAction	*act = KMainWindow::memberList->first()->actionCollection()->action("printer_test");
	if (act)
	{
		act->setEnabled(on);
		if (pr) act->setToolTip(i18n("Test printer %1").arg(pr->getName()));
		else act->setToolTip(i18n("Test printer"));
	}
}
